
UNIT LookUp;
{-- piece square tables are 128 entries long and contain the values
    for white and black in an array that allows direct one to one
    indexing for white and (index + 8) for black.
    This makes looking up values quicker than having to calculate
    the index, although some tables are symetrical so the black
    part is redundant, or not used. These values are from the
    chess programming wiki
 }

INTERFACE
USES
  Globals;

{$PACKSET 1}

TYPE
  TableType = ARRAY[SquareType] OF Int8;

CONST
  PawnPST : TableType =
          {-- white values }                  {-- black Values }
{   a1  b1   c1  d1  e1   f1  g1  h1    a1  b1   c1  d1  e1   f1  g1  h1 }
{1}( 0,  0,   0,  0,  0,   0,  0,  0,    0,  0,   0,  0,  0,   0,  0,  0,
{2}  5, 10,  10,  0,  0,  10, 10,  5,   32, 32,  32, 32, 32,  32, 32, 32,
{3}  5, -5, -10,  0,  0, -10, -5,  5,   10, 10,  20, 30, 30,  20, 10, 10,
{4}  0,  0,   0, 20, 20,   0,  0,  0,    5,  5,  10, 25, 25,  10,  5,  5,
{5}  5,  5,  10, 25, 25,  10,  5,  5,    0,  0,   0, 20, 20,   0,  0,  0,
{6} 10, 10,  20, 30, 30,  20, 10, 10,    5, -5, -10,  0,  0, -10, -5,  5,
{7} 32, 32,  32, 32, 32,  32, 32, 32,    5, 10,  10,  0,  0,  10,  10, 5,
{8}  0,  0,   0,  0,  0,   0,  0,  0,    0,  0,   0,  0,  0,   0,   0, 0
  );


  KnightPST : TableType =
  {-- encourage knights to the centre }
{    a1   b1   c1   d1   e1   f1   g1   h1 }
{1}(-50, -40, -30, -30, -30, -30, -40, -50,   0, 0, 0, 0, 0, 0, 0, 0,
{2} -40, -20,   0,   5,   5,   0, -20, -40,   0, 0, 0, 0, 0, 0, 0, 0,
{3} -30,   5,  10,  15,  15,  10,   5, -30,   0, 0, 0, 0, 0, 0, 0, 0,
{4} -30,   0,  15,  20,  20,  15,   0, -30,   0, 0, 0, 0, 0, 0, 0, 0,
{5} -30,   5,  15,  20,  20,  15,   5, -30,   0, 0, 0, 0, 0, 0, 0, 0,
{6} -30,   0,  10,  15,  15,  10,   0, -30,   0, 0, 0, 0, 0, 0, 0, 0,
{7} -40, -20,   0,   0,   0,   0, -20, -40,   0, 0, 0, 0, 0, 0, 0, 0,
{8} -50, -40, -30, -30,  -3, -30, -40, -50,   0, 0, 0, 0, 0, 0, 0, 0
  );




  SliderPST : TableType =
  {-- encourage centralizing }
{     a1   b1   c1   d1   e1   f1   g1  h1 }
{1}( -20, -10, -10, -10, -10, -10, -10, -20,   0, 0, 0, 0, 0, 0, 0, 0,
{2}  -10,   5,   0,   0,   0,   0,   5, -10,   0, 0, 0, 0, 0, 0, 0, 0,
{3}  -10,  10,  10,  10,  10,  10,  10, -10,   0, 0, 0, 0, 0, 0, 0, 0,
{4}  -10,   0,  10,  10,  10,  10,   0, -10,   0, 0, 0, 0, 0, 0, 0, 0,
{5}  -10,   5,   5,  10,  10,   5,   5, -10,   0, 0, 0, 0, 0, 0, 0, 0,
{6}  -10,   0,   5,  10,  10,   5,   0, -10,   0, 0, 0, 0, 0, 0, 0, 0,
{7}  -10,   0,   0,   0,   0,   0,   0, -10,   0, 0, 0, 0, 0, 0, 0, 0,
{8}  -20, -10, -10, -10, -10, -10, -10, -20,   0, 0, 0, 0, 0, 0, 0, 0
  );


  {-- early and middle game }
  KingPST1 : TableType =
  {-- encourage the king to stay safe in the corner }
        {-- white values }                          {-- black Values }
{    a1   b1   c1   d1   e1   f1   g1  h1     a1   b1   c1   d1   e1   f1   g1   h1 }
{1}( 20,  30,  10,   0,   0,  10,  30,  20,   -30, -40, -40, -50, -50, -40, -40, -30,
{2}  20,  20,   0,   0,   0,   0,  20,  20,   -30, -40, -40, -50, -50, -40, -40, -30,
{3} -10, -20, -20, -20, -20, -20, -20, -10,   -30, -40, -40, -50, -50, -40, -40, -30,
{4} -20, -30, -30, -40, -40, -30, -30, -20,   -30, -40, -40, -50, -50, -40, -40, -30,
{5} -30, -40, -40, -50, -50, -40, -40, -30,   -20, -30, -30, -40, -40, -30, -30, -20,
{6} -30, -40, -40, -50, -50, -40, -40, -30,   -10, -20, -20, -20, -20, -20, -20, -10,
{7} -30, -40, -40, -50, -50, -40, -40, -30,    20,  20,   0,   0,   0,   0,  20,  20,
{8} -30, -40, -40, -50, -50, -40, -40, -40,    20,  30,  10,   0,   0,  10,  30,  20
  );


  {-- end game }
  KingPST2 : TableType =
  {-- encourage king to come out to the centre }
        {-- white values }                            {-- black Values }
{     a1   b1   c1   d1   e1   f1   g1   h1     a1   b1   c1   d1   e1   f1   g1   h1 }
{1}( -50, -30, -30, -30, -30, -30, -30, -50,   -50, -40, -30, -20, -20, -30, -30, -50,
{2}  -30, -30,   0,   0,   0,   0, -30, -30,   -30, -20, -10,   0,   0, -10, -20, -30,
{3}  -30, -10,  20,  30,  30,  20, -10, -30,   -30, -10,  20,  30,  30,  20, -10, -30,
{4}  -30, -10,  30,  40,  40,  30, -10, -30,   -30, -10,  30,  40,  40,  30, -10, -30,
{5}  -30, -10,  30,  40,  40,  30, -10, -30,   -30, -10,  30,  40,  40,  30, -10, -30,
{6}  -30, -10,  20,  30,  30,  20, -10, -30,   -30, -10,  20,  30,  30,  20, -10, -30,
{7}  -30, -20, -10,   0,   0, -10, -20, -30,   -30, -30,   0,   0,   0,   0, -30, -30,
{8}  -50, -40, -30, -20, -20, -30, -40, -50,   -50, -30, -30, -30, -30, -30, -30, -30
  );





{-------------------------Some lookup data------------------------------------}
TYPE
  SetOfSqType = SET OF Byte;
  SetOfPieces = SET OF PieceType;

CONST
  BrdSqSet : SetOfSqType = [0..7, $10..$17, $20..$27, $30..$37, $40..$47, $50..$57, $60..$67, $70..$77, OffBoard];
  {-- the white squares }
  WhSqSet  : SetOfSqType = [1, 3, 5, 7, $10, $12, $14, $16, $21, $23, $25, $27, $30, $32, $34, $36,
                             $41, $43, $45, $47, $50, $52, $54, $56, $61, $63, $65, $67, $70, $72, $74, $76];
  {-- the black squares }
  BlSqSet  : SetOfSqType = [0, 2, 4, 6, $11, $13, $15, $17, $20, $22, $24, $26, $31, $33, $35, $37,
                            $40, $42, $44, $46, $51, $53, $55, $57, $60, $62, $64, $66, $71, $73, $75, $77];
  {-- the rim of the board }
  RimSqSet : SetOfSqType = [0..7, $70..$77, $10, $20, $30, $40, $50, $60, $17, $27, $37, $47, $57, $67];

  WhitePieceSet : SetOfPieces = [Wpawn..Wking];
  BlackPieceSet : SetOfPieces = [Bpawn..Bking];
  AllPieceSet   : SetOfPieces = [Wpawn..Bking];
  AllPawnSet    : SetOfPieces = [Wpawn, Bpawn];

  {-- squares that white/black can move to }
  GoodWhiteSquares : SetOfPieces = [NoPiece, Bpawn..Bking];
  GoodBlackSquares : SetOfPieces = [NoPiece, Wpawn..Wking];


{-- the piece directions for their moves. Add this to the square to get a destination }
TYPE
  DirType  = ARRAY[0..7] OF Int8;
  DeltaSet = SET OF Byte;
CONST
  RookOffSet   : DirType = (-1, 1, $10, -$10, 0, 0, 0, 0);
  BishopOffset : DirType = (-$11, -$F, $F, $11, 0 , 0, 0, 0);
  KnightOffSet : DirType = ($E, $12, $1F, $21, -$E, -$12, -$1F, -$21);
  QueenOffSet  : DirType = (-$11, -$10, -$F, -1, 1, $F, $10, $11);
  KingOffSet   : DirType = (-$11, -$10, -$F, -1, 1, $F, $10, $11);


IMPLEMENTATION

END.
