UNIT Debugging;

INTERFACE
USES
  Globals,
  GenMoves;

procedure WritePieceLists;
procedure CheckTTkey;
procedure WriteEval;
procedure WriteVars;
procedure WriteMoveList(Ply : Byte);
function  Perft(Depth : Byte) : Word64;
procedure WalkThePV(L : Byte);
procedure DoDebug;


{----------------------------------------------------------------------------}
IMPLEMENTATION
USES
  Crt,
  Zorbist,
  Utils,
  Move,
  Eval,
  Clocks,
  TheBoard;


procedure WritePieceLists;
var
  i : SquareType;
begin
  with Data[Ply] do
    begin
      Write('White pawns ');
      for i IN W_Pawns DO
        if i <> OffBoard then
          Write(DecToHex(i), ' ');
      Writeln;

      Write('Black pawns ');
      for i IN B_Pawns do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');
      Writeln;

      Write('White Rooks ');
      for i IN W_Rook do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');

      Write('Black Rooks ');
      for i IN B_Rook do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');
      Writeln;

      Write('White Knights ');
      for i IN W_Knight do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');

      Write('Black Knight ');
      for i IN B_Knight do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');
      Writeln;

      Write('White Bishops ');
      for i IN W_Bishop do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');

      Write('Black Bishops ');
      for i IN B_Bishop do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');
      Writeln;

      Write('White Queen ');
      for i IN W_Queen do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');

      Write('Black Queen ');
      for i IN B_Queen do
        if i <> OffBoard then
          Write(DecToHex(i), ' ');
      Writeln;

      Writeln('White King ', DecToHex(W_King), '  Black King ', DecToHex(B_King));

    end;

end;


PROCEDURE WriteEval;
VAR
  Material : int16;
BEGIN
  Window(1, 12, 50, 22);
  ClrScr;

  Material := CalcMaterial;

  Write('Material:', Material);
  IF Wtm THEN
    Writeln(' Positional:', Evaluation(-12345, -12345) - Material)
  ELSE
    Writeln(' Positional:', Evaluation(-12345, -12345) + Material);
  WritePieceLists;
  Write('White To Move: ', Wtm);
  Writeln(' Computer plays: ', ComputerPlays);
  Writeln('End Game = ', EndGame);
  Write('Hit Return');
  Readln;
  ClrScr;
  Window(1, 1, 80, 25);
END;


procedure CheckTTkey;
begin
  if Data[Ply].ZorbistKey <> CalcBrdKey then
    begin
      GoToXY(1, 25);
      Write('Error in Zorbist Key ! (press return)');
      Readln;
    end;
end;


procedure WriteVars;
begin
  GoToXY(1, 20);
  Writeln('Move Num: ', MoveNmbr);
  Writeln('Ply: ', Ply);
  Writeln('White To Move: ', Wtm);
  Writeln('Human move: ', HumansMove);
  Readln;
end;


procedure WriteMoveList(Ply : Byte);
var
  f : text;
  i, j : Byte;
begin
  Assign(f, 'movelist.txt');
  Rewrite(f);

  for i := 0 to Ply do
    write(f, PV[0, i], ' ');
  Writeln(f, '---------------------------');

  for j := 0 to Ply do
    begin
      Writeln(f, 'Ply: ', j, '------------------------');
      i := 1;
      while MoveList[j, i].MoveStr <> '' do
        begin
          WriteLn(f, MoveList[j, i].MoveStr, ' ', MoveList[j, i].MoveValue);
          Inc(i);
        end;
      WriteLn(f);
    end;

  Close(f);
end;



function Perft(Depth : Byte) : Word64;
{-- Since I do not do underpromotion my figures are wrong in position that have this }
var
  Nodes,
  Total  : Word64;
  M      : Str4;
  i,
  j      : Byte;
  Z      : Word64;
  MovNum : Byte;
begin
  Nodes := 0;
  Total := 0;
  j := 0;
  Z := Data[Ply].ZorbistKey;  {-- start position }

  MovNum :=  GenAllMoves('');
  if Depth = 0 then
    Total := 1
  else
    begin
      for i := 1 to MovNum do
        begin
          M := GetAMove(i);

          Inc(Ply);
          Data[Ply] := Data[Ply - 1];

          MakeMove(M, False);

          If not InCheck(not Wtm) then
            begin
              Nodes := Perft(depth - 1);
              Total := Total + Nodes;
              Inc(j);

              if Depth = 5 then
                begin
                  Write(j : 2, ' ', M, Nodes : 10, '    ');
                  if Odd(j) then
                    writeln;
                end;
            end;

          UnMakeMove;

          if Z <> Data[Ply].ZorbistKey then
            WriteLn('Zorbist Key error at depth = ', depth);

        end;
    end;

  Perft := Total;

end;


procedure WalkThePV(L : Byte);
var
  SaveData   : DataType;
  SaveMovNbr : Byte;
  Wmove,
  Hmove      : Boolean;
  Mstr       : Str4;
  i          : Byte;
begin
  {-- save state of play }
  SaveData   := Data[Ply];
  SaveMovNbr := MoveNmbr;
  Wmove      := Wtm;
  Hmove      := HumansMove;

  FOR i := 0 TO L DO
    BEGIN
      Mstr := PV[0, i];
      MakeMove(Mstr, False);
    END;

  DisplayTheBoard;
  WritePV(L);
  WriteEval;

  {-- restore state of play }
  Data[Ply]  := SaveData;
  MoveNmbr   := SaveMovNbr;
  Wtm        := Wmove;
  HumansMove := Hmove;
end;


procedure DoDebug;
begin

end;



BEGIN
END.
