UNIT Eval;

INTERFACE


FUNCTION Evaluation(Alpha, Beta : int16) : Int16;


IMPLEMENTATION
USES crt,
  Globals,
  Utils,
  LookUp,
  TT;




{---------------------- Count mobility of pieces ----------------------------}


FUNCTION SliderMobil(Col    : ColourType;       {-- to move }
                    HomeSq : SquareType;        {-- starting from this square }
                    OffSet : DirType) : Int16;  {-- array of the directions a piece can move }
{-- handles rook and bishop and queen }
{-- the squares may be attacked by the opponent, but ignore this for the moment }
VAR
  i       : 0..3;
  ToSq    : Int16;
  Pset    : SetOfPieces;
  SqCount : Int8;
BEGIN
  IF Col = WhitePieces THEN
    Pset := BlackPieceSet    {-- white to move, black capture squares }
  ELSE
    Pset := WhitePieceSet;

  SqCount := 0;
  IF HomeSq <> OffBoard THEN
    BEGIN
      FOR i := 0 TO 3 DO                     {-- each direction }
        BEGIN
          ToSq := HomeSq + OffSet[i];        {-- the first destination square }
          {-- count the vacant squares }
          WHILE OnBoard(ToSq) AND (Data[Ply].Board[ToSq] IN (Pset + [NoPiece])) DO
            BEGIN
              Inc(SqCount);
              ToSq := ToSq + OffSet[i];      {-- next square }
            END;
        END;
    END;

  SliderMobil := SqCount;
END;


FUNCTION QueenMobil(Col : ColourType ; HomeSq : SquareType) : Int16;
{-- how many squares can the queen move to }
BEGIN
  QueenMobil := 0;
  IF NOT InOpening THEN    {-- dont move the queen early }
    BEGIN
      IF HomeSq <> OffBoard THEN
        IF Col = WhitePieces THEN
          QueenMobil := SliderMobil(WhitePieces, HomeSq, RookOffset) +
                        SliderMobil(WhitePieces, HomeSq, BishopOffset)
        ELSE
          QueenMobil := SliderMobil(BlackPieces, HomeSq, RookOffset) +
                        SliderMobil(BlackPieces, HomeSq, BishopOffset);
    END;
END;


FUNCTION KnightMobil(Col : ColourType; HomeSq : SquareType) : Int16;
{-- how many squares can the knight move to }
VAR
  i       : 0..7;
  ToSq    : Int16;
  SqCount : Int8;
  Pset    : SetOfPieces;
BEGIN
  IF Col = WhitePieces THEN
    Pset := GoodWhiteSquares {-- white to move, so empty squares or black capture squares }
  ELSE
    Pset := GoodBlackSquares;

  SqCount := 0;
  IF HomeSq <> OffBoard THEN
    BEGIN
      FOR i := 0 TO 7 DO
        BEGIN
          ToSq := HomeSq + KnightOffset[i];
          IF OnBoard(ToSq) AND (Data[Ply].Board[ToSq] IN (Pset + [NoPiece])) THEN
            Inc(SqCount);
        END;
    END;

  KnightMobil := SqCount;
END;

{---------------------------------end of mobility-----------------------------}

FUNCTION PassedPawn(Col   : ColourType;               {-- who is on move }
                    Sq    : SquareType;               {-- the pawn is on this square }
                    Plist : PawnListType) : Boolean;  {-- the enemy pawns }
VAR
  i : 0..7;
  MyFyle,
  YourFyle : 0..7;
BEGIN
  MyFyle := Fyle(Sq);                     {-- the file my pawn is on }

  IF Col = WhitePieces THEN
    BEGIN
      FOR i := 0 TO 7 DO                  {-- cycle thru the enemy pawns }
        BEGIN
          YourFyle := Fyle(Plist[i]);     {-- get the file the enemy pawn is on }
          IF (Plist[i] > (Sq + 1)) THEN   {-- in front of my pawns ? }
            IF (YourFyle = MyFyle) OR (YourFyle = MyFyle - 1) OR (YourFyle = MyFyle + 1) THEN
              Exit(False);
         END;
    END
  ELSE
    BEGIN
      FOR i := 0 TO 7 DO
        IF Plist[i] <> OffBoard THEN  {-- enemy pawn }
          BEGIN
            YourFyle := Fyle(Plist[i]);
            IF (Plist[i] < (Sq - 1)) THEN    {-- blacks pawns are coming down the board so its sq - 1 }
              IF (YourFyle = MyFyle) OR (YourFyle = MyFyle - 1) OR (YourFyle = MyFyle + 1) THEN
                Exit(False);
         END;
    END;

  {-- if we get to here then it is a passed pawn }
  PassedPawn := True;
END;


FUNCTION PawnEval(Col : ColourType; W_Plist, B_Plist : PawnListType) : Int16;
{-- using the PST, and isolated/doubled/passed pawns and queen side pawn majority }
CONST
  DoubledPawnPenalty  = 10;
  IsolatedPawnPenalty = 20;
  PassedPawnBonus     = 10;
  QueenSideMajority   = 25;
  QueeningDist        : ARRAY[0..7] OF Byte = (0, 60, 40, 20, 10, 5, 0, 0);
VAR
  i            : 0..7;
  Value        : Int16;
  WPawnsOnFile,
  BPawnsOnFile : ARRAY[-1..8] OF Byte;  {-- list of how many pawns on each file, includes offboard left and right  }
  PwnHashEval  : Int16;
  FromSq       : Byte;
 BEGIN
   {-- look for a hashed result first }
   IF Col = WhitePieces THEN
     PwnHashEval := GetPawnHashValue(WhitePieces, Data[Ply].WhtPwnHashKey)
   ELSE
     PwnHashEval := GetPawnHashValue(BlackPieces, Data[Ply].BlaPwnHashKey);
   IF PwnHashEval <> 0 THEN
     Exit(PwnHashEval);

  Value := 0;
  FillChar(WPawnsOnFile, SizeOf(WPawnsOnFile), 0);
  FillChar(BPawnsOnFile, SizeOf(BPawnsOnFile), 0);

  {-- count the pawns on each file }
  FOR i := 0 TO 7 DO
    BEGIN
      IF W_Plist[i] <> OffBoard THEN
        Inc(WPawnsOnFile[Fyle(W_Plist[i])]);
       IF B_Plist[i] <> OffBoard THEN
        Inc(BPawnsOnFile[Fyle(B_Plist[i])]);
    END;

  {-- assign values via PST and check for passed pawns }
  IF Col = WhitePieces THEN
    FOR i := 0 TO 7 DO                  {-- each pawn }
      BEGIN
        IF W_Plist[i] <> OffBoard THEN  {-- a pawn is on the board }
          BEGIN
            Inc(Value, PawnPST[W_Plist[i]]);
            IF PassedPawn(Col, W_Plist[i], B_Plist) THEN
              Inc(Value, PassedPawnBonus * Rank(W_Plist[i]));   {-- the closer to queening the better }
          END;
      END
  ELSE
    FOR  i := 0 TO 7 DO
      IF B_Plist[i] <> OffBoard THEN
        BEGIN
          Inc(Value, PawnPST[B_Plist[i]+8]);
          IF PassedPawn(Col, B_Plist[i], W_Plist) THEN
            Inc(Value, PassedPawnBonus * (7 - Rank(B_Plist[i])));
        END;

  {-- scan lists for pawns on each file for doubled or isolated pawns }
  IF Col = WhitePieces THEN
    FOR i := 0 TO 7 DO
      BEGIN
        IF WPawnsOnFile[i] > 0 THEN
          BEGIN
            {-- doubled pawns }
            IF WPawnsOnFile[i] > 1 THEN
              Dec(Value, WPawnsOnFile[i]*DoubledPawnPenalty);

            {-- isolated pawns }
            IF (WPawnsOnFile[i-1] = 0) AND (WPawnsOnFile[i+1] = 0) THEN
              Dec(Value, IsolatedPawnPenalty);
          END;
      END
  ELSE
    FOR i := 0 TO 7 DO
      BEGIN
        IF BPawnsOnFile[i] > 0 THEN
          BEGIN
            {-- doubled pawns }
            IF BPawnsOnFile[i] > 1 THEN
              Dec(Value, BPawnsOnFile[i]*DoubledPawnPenalty);

            {-- isolated pawns }
            IF (BPawnsOnFile[i-1] = 0) AND (BPawnsOnFile[i+1] = 0) THEN
              Dec(Value, IsolatedPawnPenalty);
          END;
      END;

  {-- reward queen side pawn majority }
  IF Wtm THEN
    BEGIN
      IF Data[Ply].W_King > 3 THEN   {-- not castled long }
        IF (WPawnsOnFile[0] +  WPawnsOnFile[1] + WPawnsOnFile[2] + WPawnsOnFile[3] ) >
           (BPawnsOnFile[0] +  BPawnsOnFile[1] + BPawnsOnFile[2] + BPawnsOnFile[3] ) THEN
          Inc(Value, QueenSideMajority);
    END
  ELSE
    BEGIN
      IF Data[Ply].B_King > $73 THEN
        IF (BPawnsOnFile[0] +  BPawnsOnFile[1] + BPawnsOnFile[2] + BPawnsOnFile[3] ) >
           (WPawnsOnFile[0] +  WPawnsOnFile[1] + WPawnsOnFile[2] + WPawnsOnFile[3] ) THEN
         Inc(Value, QueenSideMajority);
    END;

  IF Col = WhitePieces THEN
    AddToPawnTT(WhitePieces, Data[Ply].WhtPwnHashKey, Value)
  ELSE
    AddToPawnTT(BlackPieces, Data[Ply].BlaPwnHashKey, Value);

  Assert( (Value > 128) AND (Value < -127), 'Error in Pawn eval');

  PawnEval := Value;
END;


FUNCTION KingSafety(Col : ColourType) : int16;
{-- pawn shield infront of king and knight on f3/f6 }
CONST
  Safety = 10;
VAR
  Sq    : SquareType;
  Bonus : int16;
BEGIN
  Bonus := 0;

  IF (Col = WhitePieces) AND Castled(WhitePieces) AND (Data[Ply].W_King = 6) THEN
    BEGIN
      FOR Sq := $15 TO $17 DO
        IF Data[Ply].Board[Sq] = Wpawn THEN
          Inc(Bonus, Safety);

      IF Data[Ply].Board[$25] <> Wknight THEN
        Dec(Bonus, Safety);
    END;

  IF (Col = BlackPieces) AND Castled(BlackPieces) AND (Data[Ply].B_King = $76) THEN
    BEGIN
      FOR Sq := $65 TO $67 DO
        IF Data[Ply].Board[Sq] = Bpawn THEN
          Inc(Bonus, Safety);

      IF Data[Ply].Board[$55] <> Bknight THEN
        Dec(Bonus, Safety);
    END;

  KingSafety := Bonus;
END;


FUNCTION KingEval(Col : ColourType; KingSq : SquareType) : Int16;
TYPE
  ProxType = ARRAY[0..7] OF Byte;
CONST
  CastlingValue   = 30;
  ProximetryBonus : ProxType = (0, 50, 25, 15, 12, 9, 5, 0);
VAR
  Value,
  EndGameProxBonus : Int8;
BEGIN
  Value := 0;

  IF KingSq <> OffBoard THEN
    BEGIN
      WITH Data[Ply] DO  {-- encourage king proximetry in end game }
        IF EndGame THEN
          EndGameProxBonus := ProximetryBonus[Distance(W_King, B_King)];

      IF Col = WhitePieces THEN
        BEGIN
          IF EndGame THEN
            Value := KingPST2[KingSq] + EndGameProxBonus
          ELSE
            BEGIN
              Value := KingPST1[KingSq] + KingSafety(WhitePieces);

              IF NOT Castled(WhitePieces) AND (Data[Ply].W_King <> 4) THEN {-- dont move the king before castling }
                Dec(Value, CastlingValue);
              IF NOT Data[Ply].WCanCastShort  THEN   {-- encourage castling on the short side }
                Inc(Value, CastlingValue);
            END;
        END
      ELSE   {-- black }
        BEGIN
          IF EndGame THEN
            Value := KingPST2[KingSq+8] + EndGameProxBonus
          ELSE
            BEGIN
              Value := KingPST1[KingSq+8] + KingSafety(BlackPieces);

              IF NOT Castled(BlackPieces) AND (Data[Ply].W_King <> $74) THEN
                Dec(Value, CastlingValue);
              IF Castled(BlackPieces) THEN
                  Inc(Value, CastlingValue);
            END;
        END;
    END;

  KingEval := Value;
END;


FUNCTION Evaluation(Alpha, Beta : int16) : int16;
CONST
  TwoBishopReward = 10;
  CutOff          = 200;
VAR
  Total,
  W_Eval,
  B_Eval,
  lScore : Int16;
BEGIN
  IF (Alpha <> -12345) AND (Beta <> -12345) THEN  {-- this flags that we are using the evaluation command and want the real result }
  {-- try lazy eval first  }
    IF NOT EndGame THEN
      BEGIN
        lScore := CalcMaterial;
        IF NOT Wtm THEN
          lScore := -lScore;

        IF (lScore + CutOff) < Alpha THEN
          Exit(Alpha);
        IF (lScore - CutOff) > Beta THEN
          Exit(Beta);
      END;

  {-- now the positional stuff }
  WITH Data[Ply] DO
    BEGIN
      W_Eval := KingEval   (WhitePieces, W_King) +
                PawnEval   (WhitePieces, W_Pawns, B_Pawns) +
                KnightMobil(WhitePieces, W_Knight[0]) +
                KnightMobil(WhitePieces, W_Knight[1]) +
                SliderMobil(WhitePieces, W_Rook[0], RookOffset) +
                SliderMobil(WhitePieces, W_Rook[1], RookOffset) +
                SliderMobil(WhitePieces, W_Bishop[0], BishopOffset) +
                SliderMobil(WhitePieces, W_Bishop[1], BishopOffset) +
                QueenMobil (WhitePieces, W_Queen[0]) +
                QueenMobil (WhitePieces, W_Queen[1]);

      B_Eval := KingEval   (BlackPieces, B_King) +
                PawnEval   (BlackPieces, W_Pawns, B_Pawns) +
                KnightMobil(BlackPieces, B_Knight[0]) +
                KnightMobil(BlackPieces, B_Knight[1]) +
                SliderMobil(BlackPieces, B_Rook[0], RookOffset) +
                SliderMobil(BlackPieces, B_Rook[1], RookOffset) +
                SliderMobil(BlackPieces, B_Bishop[0], BishopOffset) +
                SliderMobil(BlackPieces, B_Bishop[1], BishopOffset) +
                QueenMobil (BlackPieces, B_Queen[0]) +
                QueenMobil (BlackPieces, B_Queen[1]);

      {-- encourage keeping 2 bishops }
      IF Wtm AND (W_Bishop[1] <> OffBoard) THEN
        Inc(W_Eval, TwoBishopReward);
      IF NOT Wtm AND (B_Bishop[1] <> OffBoard) THEN
        Inc(B_Eval, TwoBishopReward);

    END;

  Total := CalcMaterial + 6*(W_Eval - B_Eval) div 10;

  {-- NegaMax must have the score from the "side to move" point of view }
  IF Wtm THEN
    Evaluation := Total
  ELSE
    Evaluation := -Total;
END;


BEGIN

END.
