 PROGRAM ltchess2;

{$ASSERTIONS ON}
{$INLINE ON}
{$RANGECHECKS ON}
{$OVERFLOWCHECKS ON}
{$BOOLEVAL OFF}
{$DEBUGINFO ON}



USES
  Crt,
  Globals,
  Init,
  TextUI,
  Zorbist,
  TT,
  Debugging,
  TheBoard,
  Opening,
  Move,
  Search,
  GenMoves,
  Utils,
  Eval,
  LookUp,
  Profile;


{------------------------------------------------------------------------------ }
PROCEDURE DoMoveCmd;
VAR
  Mrec : MoveRecType;
BEGIN
  IF Moves[MoveNmbr].MoveStr = 'MATE' THEN    {-- its check mate and there are no moves for the computer }
    SwapSides
  ELSE
    IF LegalMove(Moves[MoveNmbr].MoveStr) THEN            {-- is the entered move legal }
      BEGIN
        {-- do the accounting for the move history }
        MoveStrToMoveRec(Moves[MoveNmbr].MoveStr, Mrec);  {-- get the details from the move string }

        IF Mrec.CapturedPiece = NoPiece THEN              {-- are we capturing on the tosq }
          Inc(MovesSinceCapture)
        ELSE
          MovesSinceCapture := 0;

        {-- a pawn move or capture means there can be no 3 times repetition, so
            reset the board history to zero. Inc fifty move counter if not a capture }
        Inc(FiftyMoveCnt, Ord(Mrec.CapturedPiece = NoPiece));

        IF (Mrec.MovingPiece IN [WPawn, BPawn]) OR
           (Mrec.CapturedPiece <> NoPiece) THEN    {-- the game history is used for 3 fold rep }
          BEGIN                                   {-- it can be reset if there is a capture or pawn move }
            FillChar(GameHistoryKeys, SizeOf(GameHistoryKeys), 0);
            GameHistoryIndex := 0;
          END;

        PushMove;                                 {-- onto takeback stack }
        MakeMove(Moves[MoveNmbr].MoveStr, True);  {-- update the Data record/Board }

      END      {-- if }
    ELSE
      ErrorMsg('Illegal Move');

END;



{-----------------------------------------------------------------------------}
BEGIN
  InitGame;
  DisplayTheBoard;

  REPEAT
    {-- get a move from either the user or the computer }
    IF HumansMove THEN
      Cmd := GetCmd                {-- humans has typed in a command }
    ELSE
      Cmd := CalcMove;             {-- checks in library for a move, or else searches }

    CASE Cmd OF                    {-- process the command }
      NewGameCmd : InitGame;       {-- start a new game }
      MoveCmd    : DoMoveCmd;      {-- process the move }
      PlayCmd    : PlayMove;       {-- force computer move }
      BackCmd    : TakeBackMove;   {-- take back last move }
      ReadCmd    : ReadBoard;      {-- read a saved board from disk }
      WriteCmd   : WriteBoard;     {-- save a board position to disk }
      SetupCmd   : SetUpBoard;     {-- set a position on the board }
      EvalCmd    : WriteEval;      {-- write some evaluation info to screen, when testing }
      DebugCmd   : Debug := False; {-- true to run a debug routine }
    END; {-- case}

    DisplayTheBoard;
  UNTIL Cmd = Quitcmd;
END.
