UNIT Move;

INTERFACE
USES
  Globals;


PROCEDURE MakeMove(TheMove : Str4; Qsearch : Boolean);
PROCEDURE UnMakeMove;
PROCEDURE PlayMove; InLine;
PROCEDURE PushMove;
PROCEDURE TakeBackMove;

FUNCTION LegalMove(TheMove : Str4) : Boolean;
FUNCTION CalcMove : CmdType;


{----------------------------------------------------------------------------}
IMPLEMENTATION
USES
  Crt,
  Zorbist,
  Opening,
  Utils,
  Search,
  GenMoves,
  TT,
  LookUp;


{$ASSERTIONS ON}
{$RANGECHECKS ON}



VAR
  TakeBackStack : ARRAY[0..7] OF RECORD                 {-- allows for 8 takebacks }
                                    Data   : DataType;
                                    MovNum : Byte;
                                    Wtm    : Boolean;
                                  END;



PROCEDURE PlayMove; InLine;
{-- operator has pressed play to force the next move }
BEGIN
  IF Moves[MoveNmbr].MoveStr <> 'MATE' THEN    {-- its check mate and there are no moves }
    BEGIN
      HumansMove := NOT HumansMove;
      IF ComputerPlays = WhitePieces THEN
        ComputerPlays := BlackPieces
      ELSE
        ComputerPlays := WhitePieces;
    END;
END;


PROCEDURE PushMove;
VAR
  i : 1..7;
BEGIN
  {-- bump moves up }
  FOR i := 7 DOWNTO 1 DO
    TakeBackStack[i] := TakeBackStack[i - 1];

  {-- put new move on the stack }
  TakeBackStack[0].Data   := Data[Ply];
  TakeBackStack[0].MovNum := MoveNmbr;
  TakeBackStack[0].Wtm := Wtm;
END;


PROCEDURE TakeBackMove;
VAR
  i : 0..6;
BEGIN
  IF MoveNMbr > 1 THEN
    BEGIN
      {-- take the move off the stack }
      Data[Ply]  := TakeBackStack[0].Data;
      MoveNmbr   := TakeBackStack[0].MovNum;
      Wtm        := TakeBackStack[0].Wtm;
      HumansMove := True;

      {-- bump the moves down the stack }
      FOR i := 0 TO 6 DO
        TakeBackStack[i] := TakeBackStack[i + 1];

      Nodes  := 1;
      QNodes := 1;

      {-- zero tables }
      FillChar(PV, SizeOf(PV), 0);
      FillChar(KillerMoves, SizeOf(KillerMoves),  0);
      FillChar(HistoryTable, SizeOf(HistoryTable), 0);
      InitTTs;
      IF GameHistoryIndex > 0 THEN
        Dec(GameHistoryIndex);
    END;
END;


PROCEDURE MoveRook(ToSq : SquareType);
{-- the king gets moved in the main procedure so we just move the rook here }
BEGIN
  WITH Moves[MoveNmbr] DO
    CASE ToSq OF
      6 : BEGIN  {-- white castles short }
            {-- remove rook }
            Data[Ply].Board[7]   := NoPiece;
            Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[WRook, 7];
            RemoveFromPieceList(WRook, 7);

            {-- place rook }
            Data[Ply].Board[5]   := WRook;
            Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[WRook, 5];
            AddToPieceList(WRook, 5);
         END;

      2 : BEGIN  {-- white castles long }
            {-- remove rook }
            Data[Ply].Board[0]   := NoPiece;
            Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[WRook, 0];
            RemoveFromPieceList(WRook, 0);

            {-- place rook }
            Data[Ply].Board[3]   := WRook;
            Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[WRook, 3];
            AddToPieceList(WRook, 3);
          END;

      $76 : BEGIN  {-- black castles sort }
              {-- remove rook }
              Data[Ply].Board[$77] := NoPiece;
              Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[BRook, $77];
              RemoveFromPieceList(BRook, $77);

              {-- place rook }
              Data[Ply].Board[$75] := BRook;
              Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[BRook, $75];
              AddToPieceList(BRook, $75);
            END;

        $72 : BEGIN  {-- black castles long }
                {-- remove rook }
                Data[Ply].Board[$70] := NoPiece;
                Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[BRook, $70];
                RemoveFromPieceList(BRook, $70);

                {-- place rook }
                Data[Ply].Board[$73] := BRook;
                Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[BRook, $73];
                AddToPieceList(BRook, $73);
              END;
    END;  {-- case }
END;


PROCEDURE MovePiece(TheMove : Str4);
{-- using TheMove to update the Board and piece lists }
VAR
  MoveRec : MoveRecType;
BEGIN
  Assert(Ply < MaxPly + 1, 'error proc MakeMove');
  Assert(MoveNmbr < MaxGameMoves, 'error proc MakeMove: MoveNmbr > 1024');

  MoveStrToMoveRec(TheMove, MoveRec);  {-- converts the 4 char move string to a full discription of the move }
  WITH MoveRec DO
    BEGIN
      {-- Do pawn hash stuff }
      IF MovingPiece = Wpawn THEN
        Data[Ply].WhtPwnHashKey := Data[Ply].WhtPwnHashKey XOR WhtPawnBrd[FromSq];
      IF MovingPiece = Bpawn THEN
        Data[Ply].BlaPwnHashKey := Data[Ply].BlaPwnHashKey XOR BlaPawnBrd[FromSq];

      {-- remove the moving piece }
      Data[Ply].Board[FromSq] := NoPiece;
      RemoveFromPieceList(MovingPiece, FromSq);
      Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[MovingPiece, FromSq];

      {-- remove the captured piece if any }
      IF NOT Enpasaunt THEN
        BEGIN
          {-- Do pawn hash stuff }
          IF CapturedPiece = Wpawn THEN
            Data[Ply].WhtPwnHashKey := Data[Ply].WhtPwnHashKey XOR WhtPawnBrd[ToSq];
          IF CapturedPiece = Bpawn THEN
            Data[Ply].BlaPwnHashKey := Data[Ply].BlaPwnHashKey XOR BlaPawnBrd[ToSq];

          Data[Ply].Board[ToSq] := NoPiece;
          RemoveFromPieceList(CapturedPiece, ToSq);
          Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[CapturedPiece, ToSq];
        END
      ELSE { -- its enpasaunt }
        IF Wtm THEN
          BEGIN
            {-- remove the pawn }
            Data[Ply].BlaPwnHashKey := Data[Ply].BlaPwnHashKey XOR BlaPawnBrd[ToSq - $10];

            Data[Ply].Board[ToSq - $10] := NoPiece;
            RemoveFromPieceList(BPawn, ToSq - $10);

            Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[BPawn, ToSq - $10];
            Data[Ply].EnpasauntSq := OffBoard;
          END
        ELSE {-- blacks move }
          BEGIN
            {-- remove the pawn }
            Data[Ply].WhtPwnHashKey := Data[Ply].WhtPwnHashKey XOR WhtPawnBrd[ToSq + $10];

            Data[Ply].Board[ToSq + $10] := NoPiece;
            RemoveFromPieceList(WPawn, ToSq + $10);

            Data[Ply].ZorbistKey := Data[Ply].ZorbistKey XOR ZorbistBrd[WPawn, ToSq + $10];
            Data[Ply].EnpasauntSq := OffBoard;
          END;

      IF Promotion THEN   {-- change the piece to a queen before placing it }
        IF Wtm THEN       {-- does not do under promotion yet }
          MovingPiece := Wqueen
        ELSE
          MovingPiece := Bqueen;

      {-- account for castling rights }
      CASE MovingPiece OF
        Wking : BEGIN
                  Data[Ply].WcanCastShort := False;
                  Data[Ply].WcanCastLong  := False;
                END;
        Bking : BEGIN
                  Data[Ply].BcanCastShort := False;
                  Data[Ply].BcanCastLong  := False;
                END;
        Wrook : CASE MoveRec.FromSq OF
                  0 : Data[Ply].WcanCastLong  := False;
                  7 : Data[Ply].WcancastShort := False;
                END;
        Brook : CASE MoveRec.FromSq OF
                  $70 : Data[Ply].BcanCastLong  := False;
                  $77 : Data[Ply].BcanCastShort := False;
                END;
       END; {-- case }

      IF Castling THEN
        MoveRook(ToSq);  {-- does the rook part of castling here }

      {-- place piece }
      Data[Ply].Board[ToSq] := MovingPiece;
      Data[Ply].ZorbistKey  := Data[Ply].ZorbistKey XOR ZorbistBrd[MovingPiece, ToSq];

      {-- do pawn hash stuff }
      IF MovingPiece = Wpawn THEN
        Data[Ply].WhtPwnHashKey := Data[Ply].WhtPwnHashKey XOR WhtPawnBrd[ToSq];
      IF MovingPiece = Bpawn THEN
        Data[Ply].BlaPwnHashKey := Data[Ply].BlaPwnHashKey XOR BlaPawnBrd[ToSq];

      AddToPieceList(MovingPiece, ToSq);

      {-- flag a potential enpasaunt capture allowed on next move  }
      IF ( (MovingPiece = Wpawn) AND (FromSq IN [$10..$17]) AND (ToSq IN [$30..$37]) ) OR
         ( (MovingPiece = Bpawn) AND (FromSq IN [$60..$67]) AND (ToSq IN [$40..$47]) ) THEN
        Data[Ply].EnpasauntSq := ToSq
      ELSE
        Data[Ply].EnpasauntSq := OffBoard;

    END; {-- with }
END;


PROCEDURE MakeMove(TheMove : Str4; Qsearch : Boolean);
BEGIN
  Inc(Nodes);        {-- total nodes }
  If Qsearch THEN    {-- Q nodes }
    Inc(Qnodes);

  MovePiece(TheMove);
  IncMoveNmbr;

  {-- save the board key to the list of all moves in the game }
  GameHistoryKeys[GameHistoryIndex] := Data[Ply].ZorbistKey;
  IF GameHistoryIndex < MaxGameMoves THEN
    Inc(GameHistoryIndex);

  {-- account for whos move it is }
  SwapSides;
END;


PROCEDURE UnMakeMove;
BEGIN
  Dec(Ply);

  {-- clear the move }
  FillChar(Moves[MoveNmbr], SizeOf(Moves[MoveNmbr]), 0);
  DecMoveNmbr;

  GameHistoryKeys[GameHistoryIndex] := 0;
  IF GameHistoryIndex > 0 THEN
    Dec(GameHistoryIndex);

  {-- account for whos move it is }
  SwapSides;
END;


FUNCTION LegalMove(TheMove : Str4) : Boolean;
{-- check if the humans move is legal by generating all moves and checking if }
{-- the move is one of the moves generated and does not leave you still in check }
VAR
  i,
  NumMoves : Byte;
BEGIN
  LegalMove := False;

  NumMoves := GenAllMoves('');

  FOR i := 1 TO NumMoves DO
    IF TheMove = GetAMove(i) THEN {-- found the move made }
      BEGIN
        Inc(Ply);
        Data[Ply] := Data[Ply - 1];
        MakeMove(TheMove, False);

        IF NOT InCheck(NOT Wtm) THEN
          LegalMove := True;  {-- if still in check then this move is not legal }

        UnMakeMove;

      END;  {-- if }
END;


FUNCTION CalcMove : CmdType;
{-- computers turn, so find a move }
VAR
  MoveString : Str4;
BEGIN
  IF InLibrary THEN                           {-- find a move from the opening library }
    BEGIN
      MoveString := LibraryMove(OpeningStr);  {-- returns next move from the library}
      OpeningStr := OpeningStr + MoveString;  {-- for display }
      InLibrary  := MoveString <> '????';
      IF InLibrary THEN
        Moves[MoveNmbr].MoveStr := MoveString;  {-- fill the move }
    END;

  IF NOT InLibrary THEN                         {-- get the computers move by calling the search routine }
    Moves[MoveNmbr].MoveStr := CalcCompMove;

  Window(1, 1, 80, 25);
  GoToXY(49, 14);
  Write('My Move:', Moves[MoveNmbr].MoveStr : 5, '                   ');

  IF Moves[MoveNmbr].MoveStr = 'MATE' THEN      {-- must have been check mate }
    BEGIN
      SwapSides;
      CalcMove := NoCmd;
    END
  ELSE
    CalcMove := MoveCmd;
END;



BEGIN
END.
