UNIT Opening;

INTERFACE
USES
  Globals;

VAR
  InLibrary  : Boolean = True;
  OpeningStr : String  = '';


PROCEDURE InitOpeningLib;
FUNCTION  LibraryMove(PlayedMoves : String) : Str4;


{------------------------------------------------------------------}

IMPLEMENTATION
USES
  Crt,
  Utils;


CONST
  MaxOpenings = 73479;        {-- in the library }
  LibFileName = 'book.txt';

VAR
  {-- indexes to the start and end of the opening in the library file }
  LibStartLine,
  LibFinishLine : Word32;


PROCEDURE InitOpeningLib;
BEGIN
  InLibrary     := True;
  OpeningStr    := '';
  LibStartLine  := 1;            {-- start of file }
  LibFinishLine := MaxOpenings;  {-- end of file }
END;


FUNCTION LibraryMove(PlayedMoves : String) : Str4;
{-- return an opening move if found in the library, using the line so far
    else returns '????' }
VAR
  f          : Text;   {-- text file of openings }
  TempStr    : Str4;
  LibraryStr : String; {-- opening from the library }
  i          : Word32; {-- for loop counting }


  FUNCTION OpeningFound : Boolean;
  VAR
    TempStr : String;
  BEGIN
    TempStr := Copy(LibraryStr, 1, Length(PlayedMoves));  {-- truncate opening to match moves so far }
    OpeningFound := TempStr = PlayedMoves;
  END;


  FUNCTION WhitesFirstMove : Str4;
  {-- if human playing black, then find an opening move for white }
  CONST
    {-- some common first moves }
    WhitesFirst : ARRAY[1..16] OF Str4 =
                  ('E2E4', 'E2E4', 'E2E4', 'E2E4',  {-- more likely to be picked }
                   'D2D4', 'D2D4', 'D2D4', 'D2D4',
                   'C2C4', 'C2C4',
                   'G1F3', 'G1F3',
                   'B2B3', 'F2F4', 'G2G3', 'E2E3');
  BEGIN
    Randomize;
    WhitesFirstMove := WhitesFirst[Round(Random(16)) + 1];
  END;


BEGIN
  IF PlayedMoves = '' THEN         {-- computer plays white }
    LibraryMove := WhitesFirstMove
  ELSE
    BEGIN
      LibraryMove := '????';   {-- default value }

      Assign(f, LibFileName);
    {$I-}  Reset(f); {$I+}
      IF IOresult <> 0 THEN
        ErrorMsg('Missing Library File')
      ELSE
        BEGIN
          LibraryStr := '';
          {-- find first entry of the opening line }
          FOR i := 2 TO LibStartLine DO  {-- go to first entry that has already been found }
            Readln(f);

          WHILE NOT Eof(f) AND
                NOT OpeningFound AND
                    (LibStartLine <= LibFinishLine) DO
            BEGIN
              Readln(f, LibraryStr);
              Inc(LibStartLine);
            END;
          Dec(LibStartLine);

          {-- now find the last line of the opening played so far }
          LibFinishLine := LibStartLine;
          IF LibFinishLine < MaxOpenings THEN   {-- found an opening line }
            BEGIN
              WHILE OpeningFound DO
                BEGIN
                  Readln(f, LibraryStr);
                  Inc(LibFinishLine);
                END;
              Dec(LibFinishLine);

              {-- we now have a range of opening lines to select from }
              IF LibStartLine <= LibFinishLine THEN  {-- we have an opening }
                BEGIN
                  Randomize;
                  {-- choose a random opening from this group }
                  i := LibStartLine + Random(LibFinishLine - LibStartLine);
                  Reset(f);
                  FOR i := 1 TO (i - 1) DO  {-- skip to random line }
                    Readln(f);
                  Readln(f, LibraryStr);
                  TempStr := Copy(LibraryStr, Length(PlayedMoves) + 1, 4);
                  IF TempStr <> '' THEN
                    LibraryMove := TempStr;

                  {-- write the opening to the screen }
                  GoToXY(2, 23);
                  ClrEol;
                  GoToXY(2, 23);
                  Write('Opening: ');
                  {-- now write the opening moves }
                  i := 0;
                  WHILE (Length(LibraryStr) > 0) AND (i < 13) DO
                    BEGIN
                      Write(Copy(LibraryStr, 1, 4), ' ');
                      Delete(LibraryStr, 1, 4);
                      Inc(i);
                    END;
                END; {-- if }

            END; {-- if }
          Close(f);
        END; {-- else }
    END; {-- else }
END;



BEGIN
END.
