UNIT TheBoard;

{-- board is implemented as 0X88, bits 0,1,2 is the file and 4,5,6 is
    the rank. Bits 3 and 7 flag off board.

    In Hex

    8 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F
    7 60 61 62 63 64 65 66 67  .  .  .  .  .  .  . 6F
    6 50 51 52 53 54 55 56 57  .  .  .  .  .  .  . 5F
    5 40 41 42 43 44 45 46 47  .  .  .  .  .  .  . 4F
    4 30 31 32 33 34 35 36 37  .  .  .  .  .  .  . 3F
    3 20 21 22 23 24 25 26 27  .  .  .  .  .  .  . 2F
    2 10 11 12 13 14 15 16 17  .  .  .  .  .  .  . 1F
    1 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F.
      a  b  c  d  e  f  g  h

--}

INTERFACE



PROCEDURE DisplayTheBoard;
PROCEDURE ReadBoard;
PROCEDURE WriteBoard;
PROCEDURE SetUpBoard;


{----------------------------------------------------------------------------}
IMPLEMENTATION
USES
  Crt,
  Globals,
  Zorbist,
  TT,
  Utils,
  Opening,
  LookUp;

{$ASSERTIONS ON}



PROCEDURE DisplayTheBoard;
CONST
  PieceChar : String[12] = 'PNBRQKpnbrqk';
VAR
  Rank,
  Fyle    : 1..8;
  Square  : SquareType;
  Swap    : Boolean;
BEGIN
  HighVideo;
  TextBackGround(Black);
  Window(1, 1, 27, 10);

  GoToXY(1, 2);
  Swap := False;
  FOR Rank := 8 DOWNTO 1 DO
    BEGIN
      TextBackGround(Black);
      TextColor(White);
      Write(Rank : 2);

      FOR Fyle := 1 TO 8 DO
        BEGIN
          IF NOT Swap THEN
            TextBackGround(Green)
          ELSE
            TextBackGround(Yellow);

          Square := 16*(Rank - 1) + (Fyle - 1);       {-- calc' square number }
          IF Ord(Data[Ply].Board[Square]) < 7 THEN    {-- white pieces }
            TextColor(White)
          ELSE
            TextColor(Black);
          IF Data[Ply].Board[Square] = NoPiece THEN
            Write('   ')
          ELSE  {-- index the char in piecechar }
            Write(PieceChar[Ord(Data[Ply].Board[Square])] : 2, ' ');
          Swap := NOT Swap;
        END;  {-- for }
      Writeln;
      Swap := NOT Swap;
    END; {-- for }

  TextBackGround(Black);
  TextColor(White);

  Write  ('   a  b  c  d  e  f  g  h');

  {-- now write a menu }
  Window(30, 2, 48, 13);
  ClrScr;
  Writeln('n: New Game');
  Writeln('b: TakeBack');
  Writeln('p: Play    ');
  Writeln('s: Setup   ');
  Writeln('w: Write game');
  Writeln('r: Read game');
  Writeln('t: Move Time(tx)');
  WriteLn('x: Fixed Ply(xx)');
  Write  ('q: Quit');
  Window(1, 1, 80, 25);

  GoToXY(6, 11);
  IF FixedDepth < 255 THEN
    Write('Fixed Depth = ', FixedDepth, '     ')
  ELSE
    Write('Move Time = ', MoveTime, ' secs    ');

  GoToXY(6, 13);
  IF MovesSinceCapture > 99 THEN
     Write('Draw: 50 Move Rule' : 10)
  ELSE
    IF ThreeFoldDraw THEN
      Write('Draw: Three Fold Repitition')
    ELSE
      IF OnlyKingsLeft THEN
        Write('Draw: Insufficient material')
      ELSE
        Write('Moves without capture: ', MovesSinceCapture);

  LowVideo;
END;


PROCEDURE WriteBoard;
{-- writes the board position to file in almost FEN format }
CONST
  PieceChar : ARRAY[PieceType] OF Char = '?PNBRQKpnbrqk';
VAR
  f       : Text;                     {-- for board position }
  Files,
  Ranks,
  Square,
  Vacant : Byte;
BEGIN
  Assign(f, 'game.fen');
  Rewrite(f);
  FOR Ranks := 7 DOWNTO 0 DO
    BEGIN
      Vacant := 0;
      FOR Files := 0 TO 7 DO
        BEGIN
          Square := 16*Ranks + Files;
          IF Data[Ply].Board[Square] = NoPiece THEN
            Inc(Vacant)
          ELSE
            BEGIN
              IF Vacant <> 0 THEN
                BEGIN
                  Write(f, Char(Vacant + 48));
                  Vacant := 0;
                END;
              Write(f, PieceChar[Data[Ply].Board[Square]]);
            END;
          IF (Vacant <> 0) AND (Files = 7) THEN
            Write(f, Char(Vacant + 48));
        END;
      IF Ranks > 0 THEN
        Write(f, '/');
    END;

  IF Wtm THEN
    Write(f, ' w ')
  ELSE
    Write(f, ' b ');

  WITH Data[Ply] DO
    BEGIN
      IF WcanCastShort THEN
        Write(f, 'K')
      ELSE
        Write(f, '-');
      IF WcanCastLong THEN
        Write(f, 'Q')
      ELSE
        Write(f, '-');
      IF BcanCastShort THEN
        Write(f, 'k')
      ELSE
        Write(f, '-');
      IF BcanCastLong THEN
        Write(f, 'q')
      ELSE
        Write(f, '-');
    END;

  IF NOT HumansMove THEN
    Write(f, ' cm');

  IF ComputerPlays = WhitePieces THEN
    Write(f, ' cw')
  ELSE
    Write(f, ' cb');

  Close(f);
END;


PROCEDURE ReadBoard;
{-- reads the board position from file in almost FEN format }
VAR
  f       : Text;       {-- board position }
  Ch      : Char;
  S       : String[10];
  Piece   : PieceType;
  i,
  Files,
  Ranks,
  Square  : Byte;
BEGIN
  MoveNmbr := 1;
  Ply := 0;

  Assign(f, 'game.fen');
{$I-}  Reset(f);  {$I+}
  IF IOResult = 0 THEN
    BEGIN
      WITH Data[Ply] DO
        BEGIN
          FillChar(Board, SizeOf(Board), 0);

          FOR i := 0 TO 1 DO
            BEGIN
              W_Queen[i]  := OffBoard;
              B_Queen[i]  := OffBoard;
              W_Rook[i]   := OffBoard;
              B_Rook[i]   := OffBoard;
              W_Bishop[i] := OffBoard;
              B_Bishop[i] := OffBoard;
              W_Knight[i] := OffBoard;
              B_Knight[i] := OffBoard;
            END;
          FOR i := 0 TO 7 DO
            BEGIN
              W_Pawns[i] := OffBoard;
              B_Pawns[i] := OffBoard;
            END;

          Ranks := 8;
          REPEAT
            Dec(Ranks);
            Files := 0;
            REPEAT
              Read(f, Ch);
              Piece := NoPiece;

              IF Ch IN ['1'..'8'] THEN
                Inc(Files, (Ord(Ch) - 48) - 1)  {-- increments 0..7 }
              ELSE
                CASE Ch OF
                  'R' : Piece := WRook;
                  'N' : Piece := WKnight;
                  'B' : Piece := WBishop;
                  'Q' : Piece := WQueen;
                  'K' : Piece := WKing;
                  'P' : Piece := WPawn;
                  'r' : Piece := BRook;
                  'n' : Piece := BKnight;
                  'b' : Piece := BBishop;
                  'q' : Piece := BQueen;
                  'k' : Piece := BKing;
                  'p' : Piece := BPawn;
                END; {case}

              Square := 16*Ranks + Files;
              IF Piece <> NoPiece THEN
                BEGIN
                  Board[Square] := Piece;
                  CASE Piece OF
                    WKing   : W_King := Square;
                    BKing   : B_King := Square;
                    WQueen  : IF W_Queen[0] = OffBoard THEN
                                W_Queen[0] := Square
                              ELSE
                                W_Queen[1] := Square;
                    BQueen  : IF B_Queen[0] = OffBoard THEN
                                B_Queen[0] := Square
                              ELSE
                                B_Queen[1] := Square;
                    WRook   : IF W_Rook[0] = OffBoard THEN
                                W_Rook[0] := Square
                              ELSE
                                W_Rook[1] := Square;
                    BRook   : IF B_Rook[0] = OffBoard THEN
                                B_Rook[0] := Square
                              ELSE
                                B_Rook[1] := Square;
                    WKnight : IF W_Knight[0] = OffBoard THEN
                                W_Knight[0] := Square
                              ELSE
                                W_Knight[1] := Square;
                    BKnight : IF B_Knight[0] = OffBoard THEN
                                B_Knight[0] := Square
                              ELSE
                                B_Knight[1] := Square;
                    WBishop : IF W_Bishop[0] = OffBoard THEN
                                W_Bishop[0] := Square
                              ELSE
                                W_Bishop[1] := Square;
                    BBishop : IF B_Bishop[0] = OffBoard THEN
                                B_Bishop[0] := Square
                              ELSE
                                B_Bishop[1] := Square;
                    WPawn   : BEGIN
                                i := 0;
                                WHILE W_Pawns[i] <> OffBoard DO  {-- find the 1st slot }
                                  Inc(i);
                                W_Pawns[i] := Square;
                              END;
                    BPawn   : BEGIN
                                i := 0;
                                WHILE B_Pawns[i] <> OffBoard DO
                                  Inc(i);
                                B_Pawns[i] := Square;
                              END;
                  END;
                END;
              Inc(Files);
            UNTIL Files = 8;
            Read(f, Ch);         {-- '/' char }
          UNTIL Ranks = 0;

        ReadLn(f, S);

        Wtm := Pos('w ', S)  <> 0;

        WITH Data[Ply] DO
          BEGIN
            WcanCastShort := Pos('K', S) <> 0;
            WcanCastLong  := Pos('Q', S) <> 0;
            BcanCastShort := Pos('k', S) <> 0;
            BcanCastLong  := Pos('q', S) <> 0;
          END;

        HumansMove := Pos(' cm', S) = 0;

        IF Pos(' cw', S) <> 0 THEN
          ComputerPlays := WhitePieces
        ELSE
          ComputerPlays := BlackPieces;

        Enpasauntsq   := OffBoard;
        ZorbistKey    := CalcBrdKey;
        WhtPwnHashKey := CalcWhtPwnBrdKey;
        BlaPwnHashKey := CalcBlaPwnBrdKey;

      END;  {-- with }

      Close(f);
      InLibrary := False;
    END {if}
  ELSE
    ErrorMsg('No Board File Found');

  FillChar(Moves, SizeOf(Moves), 0);
  InitTTs;
  GameHistoryIndex  := 0;
  MovesSinceCapture := 0;
  FiftyMoveCnt      := 0;
  MoveNmbr          := 1;
END;


PROCEDURE SetUpBoard;
{-- to place a piece...'wbd4' = white bishop on D4
    to clear a square..'e5' = clear square E5
    to set the colour for the next move...'wm' or 'bm' (WhiteMove or BlackMove
    to quit.... 'q' }
VAR
  i          : 1..4;
  InStr      : String[4];
  Piece      : PieceType;
  BoardIndex : Int8;


  PROCEDURE WriteSetupMenu;
  BEGIN
    GoToXY(1, 15);
    Writeln(' c:    clear board       ');
    Writeln(' wnf3: white knight on f3');
    Writeln(' g1:   clear square g1   ');
    Writeln(' wm:   white to move     ');
    Writeln(' q:    quit              ');
  END;


BEGIN
  ClrScr;

  WITH Data[Ply] DO
    REPEAT
      DisplayTheBoard;
      WriteSetupMenu;

      {-- get the user  input }
      GoToXY(31, 12);
      Write('Command:                    ');
      GoToXY(40, 12);
      Readln(InStr);

      FOR i := 1 TO Length(InStr) DO
        InStr[i] := UpCase(InStr[i]);

      {-- clear board }
      IF InStr = 'C' THEN
        ClearBoard;

      IF Length(Instr) = 2 THEN
        BEGIN
          IF Instr = 'WM' THEN
            Wtm := True;
          IF Instr = 'BM' THEN
            Wtm := False;
          IF (Instr <> 'WM') AND (Instr <> 'BM') THEN
            Instr := '__' + Instr;
        END;

      {-- get the piece type and colour }
      IF Length(Instr) = 4 THEN
        BEGIN
          Piece := NoPiece;
          CASE InStr[1] OF
            'W' : CASE InStr[2] OF
                    'P' : Piece := WPawn;
                    'R' : Piece := WRook;
                    'N' : Piece := WKnight;
                    'B' : Piece := WBishop;
                    'Q' : Piece := WQueen;
                    'K' : Piece := WKing;
                  END; {case}
            'B' : CASE InStr[2] OF
                    'P' : Piece := BPawn;
                    'R' : Piece := BRook;
                    'N' : Piece := BKnight;
                    'B' : Piece := BBishop;
                    'Q' : Piece := BQueen;
                    'K' : Piece := BKing;
                  END; {case}
          END; {case}

          {-- insert or remove a piece }
          BoardIndex := CoOrdOfToSq(InStr);
          IF ( (Piece = WPawn) AND (BoardIndex < $10) ) OR
             ( (Piece = BPawn) AND (BoardIndex > $67) ) THEN
             ErrorMsg('Illegal position')
          ELSE  {-- legal placement }
            IF OnBoard(BoardIndex) THEN
              IF Board[BoardIndex] = NoPiece THEN  {-- room to insert a piece }
                BEGIN
                  Board[BoardIndex] := Piece;
                  AddToPieceList(Piece, BoardIndex);
                END
              ELSE                                  {-- just entered 'g5' so remove the piece }
                IF Board[BoardIndex] <> NoPiece THEN
                  BEGIN
                    Piece := Board[BoardIndex];
                    RemoveFromPieceList(Piece, BoardIndex);
                    Board[BoardIndex] := NoPiece;
                  END;
        END; {-- if }
    UNTIL InStr = 'Q';

  FillChar(Moves, SizeOf(Moves), 0);
  MoveNmbr := 1;
  HumansMove := True;

  ClrScr;
  Data[Ply].ZorbistKey    := CalcBrdKey;
  Data[Ply].WhtPwnHashKey := CalcWhtPwnBrdKey;
  Data[Ply].BlaPwnHashKey := CalcBlaPwnBrdKey;
  InitTTs;
  InLibrary := False;
END;


{----------------------------------------------------------------------------}
BEGIN
END.
