UNIT TT;

INTERFACE
USES
  Globals;

{$BITPACKING ON}
{$PACKENUM 1}
{$PACKRECORDS 1}

CONST
  NumTTentries  = 65536000;
  PawnTTentries =  1048576;

TYPE
  NodeType = (None, Exact, FailLow, FailHigh);
  TTtype   = BITPACKED RECORD
                      Key   : Word64;     {-- of the board position }
                      Score : Int16;      {-- the score found in this position }
                      Node  : NodeType;   {-- what type of node was it }
                      Draft : Byte;       {-- how far to the leaf...the longer the better }
                      Move  : Str4;       {-- the best move }
                    END;


  PawnTTtype = BITPACKED RECORD
                           BoardHash  : Word64;
                           Value      : Int16;
                         END;





VAR
  TransTable : ARRAY[0..NumTTentries] OF TTtype; {-- the transposition table }
  WhtPawnTT,
  BlaPawnTT  : ARRAY[0..PawnTTentries] OF PawnTTtype;

PROCEDURE InitTTs;                       {-- zeros the TT }

PROCEDURE AddToTT(Key   : Word64;       {-- of the board position }
                  Score : Int16;        {-- the score found in this position }
                  Node  : NodeType;     {-- what type of node was it }
                  D     : Byte;         {-- draft...how far to the leaf...the longer the better }
                  Move  : Str4);        {-- the best move }


FUNCTION GetFromTT(VAR Entry : TTtype) : Boolean;
{-- returns true if a hit and fills in the node data to Entry }

PROCEDURE AddToPawnTT(Col      : ColourType;
                      PawnHash : Word64;
                      Value    : Int16);


FUNCTION GetPawnHashValue(Col      : ColourType;
                          PawnHash : Word64) : Int16;



{-----------------------------------------------------------------------------}
IMPLEMENTATION


PROCEDURE InitTTs;
BEGIN
  FillChar(TransTable, SizeOf(TransTable), 0);
  FillChar(WhtPawnTT,  SizeOf(WhtPawnTT),  0);
  FillChar(BlaPawnTT,  SizeOf(BlaPawnTT),  0);
END;


PROCEDURE AddToTT(Key   : Word64;     {-- of the board position }
                  Score : Int16;      {-- the score found in this position }
                  Node  : NodeType;   {-- what type of node was it }
                  D     : Byte;       {-- draft....how far to the leaf...the longer the better }
                  Move  : Str4);
VAR
  Slot : Word64;
BEGIN
  IF NOT Wtm THEN                            {-- account for whos move it is }
    Key := NOT Key;

  Slot := Key MOD NumTTentries;              {-- find a slot }

  IF D >= TransTable[Slot].Draft THEN        {-- better infomation }
    BEGIN
      TransTable[Slot].Key   := Key;
      TransTable[Slot].Score := Score;
      TransTable[Slot].Node  := Node;
      TransTable[Slot].Draft := D;
      TransTable[Slot].Move  := Move;
    END;
END;


FUNCTION GetFromTT(VAR Entry : TTtype) : Boolean;
{-- returns the node info in "entry" }
VAR
  Slot : Word64;
BEGIN
  IF NOT Wtm THEN
    Entry.Key := NOT Entry.Key;           {-- Different key depending on whos move it is }

  Slot := Entry.Key MOD NumTTentries;

  WITH Entry DO
    IF Key = TransTable[Slot].Key THEN    {-- a correct match }
      BEGIN
        Score := TransTable[Slot].Score;
        Node  := TransTable[Slot].Node;
        Draft := TransTable[Slot].Draft;
        Move  := TransTable[Slot].Move;

        GetFromTT := True;
      END
    ELSE
      BEGIN
        Score := 0;
        Node  := None;
        Draft := 0;
        Move  := '';

        GetFromTT := False;
      END;
END;


PROCEDURE AddToPawnTT(Col      : ColourType;
                      PawnHash : Word64;
                      Value    : Int16);
VAR
  Index : Word32;
BEGIN
  Index := PawnHash MOD PawnTTentries;
  IF Col = WhitePieces THEN
    BEGIN
      WhtPawnTT[Index].BoardHash := PawnHash;
      WhtPawnTT[Index].Value     := Value;
    END
  ELSE
    BEGIN
      BlaPawnTT[Index].BoardHash := PawnHash;
      BlaPawnTT[Index].Value     := Value;
    END;
END;


FUNCTION GetPawnHashValue(Col      : ColourType;
                          PawnHash : Word64) : Int16;
VAR
  Index : Word32;
BEGIN
  GetPawnHashValue := 0;  {-- unless proven otherwise }

  Index := PawnHash MOD PawnTTentries;
  IF Col = WhitePieces THEN
    BEGIN
      IF (WhtPawnTT[Index].BoardHash = PawnHash) THEN
        GetPawnHashValue := WhtPawnTT[Index].Value;
    END
  ELSE
    BEGIN
      IF (BlaPawnTT[Index].BoardHash = PawnHash) THEN
        GetPawnHashValue := BlaPawnTT[Index].Value;
    END;
END;


BEGIN

END.
