UNIT Utils;

{-- many of the utility routines }

INTERFACE
USES
  Globals;



FUNCTION OnBoard(Sq : Int16) : Boolean; InLine;
FUNCTION Rank(Sq : SquareType) : Byte; InLine;
FUNCTION Fyle(Sq : SquareType) : Byte; InLine;
FUNCTION BoardSquareToString(Sq : Int8) : Str2; InLine;
FUNCTION CoOrdOfFromSq(TheMove : Str4) : Int8; InLine;
FUNCTION CoOrdOfToSq(TheMove  : Str4) : Int8; InLine;
FUNCTION Distance(Sq1, Sq2 : Int8) : Byte;
FUNCTION DistanceToQueening(Sq : SquareType; Col : ColourType) : Byte;
FUNCTION ClearVector(FromSq, ToSq : SquareType) : Boolean;
FUNCTION Attacked(TheSq : SquareType; ByColor : ColourType) : Boolean;
FUNCTION InCheck(IsWhite : Boolean) : Boolean;
FUNCTION IsDiagonal(FromSq, ToSq : SquareType) : Boolean; InLine;
FUNCTION ValidMove(M : Str4) : Boolean;
FUNCTION MovingPiece(Mstr : Str4) : PieceType; Inline;
FUNCTION ColourToMove(WhitesTurn : Boolean) : ColourType; InLine;
FUNCTION DecToHex(d : SquareType) : Str3;
FUNCTION ThreeFoldDraw : Boolean;
FUNCTION PVfromTT(D : Byte) : String;
FUNCTION WhiteMaterial : int16;
FUNCTION BlackMaterial : int16;
FUNCTION NumOfPawns(Col : ColourType) : Byte;
FUNCTION OnlyPawns(Wtm : Boolean) : Boolean;
FUNCTION Castled(Col : ColourType) : Boolean; InLine;
FUNCTION OnlyKingsLeft : Boolean;
FUNCTION GetEndGame    : Boolean;
FUNCTION CalcMaterial  : int16;
FUNCTION AverageEBF(VAR Nodes : SaveNodesType; Length : Byte) : Real; InLine;
FUNCTION InOpening : Boolean; Inline;


PROCEDURE MoveStrToMoveRec(Mstr : Str4; VAR Mrec : MoveRecType);
PROCEDURE SwapSides; InLine;
PROCEDURE IncMoveNmbr; InLine;
PROCEDURE DecMoveNmbr; InLine;
PROCEDURE ErrorMsg(S : String);
PROCEDURE AddToKillers(Ply : Byte; TheMove : MoveType);
PROCEDURE AddToPieceList(Piece : PieceType; Sq : SquareType);
PROCEDURE RemoveFromPieceList(Piece : PieceType; Sq : SquareType);
PROCEDURE ClearBoard;
(*PROCEDURE ShufflePV;*)
PROCEDURE WritePV(Len : Byte);
PROCEDURE DebugPV;
PROCEDURE AlterPieceValues(C : Char);
PROCEDURE AddToHistory(Move : Str4; Depth : int8);
PROCEDURE WriteHistoryTable;




{-----------------------------------------------------------------------------}
IMPLEMENTATION
USES
  Crt,
  LookUp,
  TT,
  Move,
  Opening;

{$ASSERTIONS ON}
{$INLINE ON}


FUNCTION DecToHex(d : SquareType) : Str3;
TYPE
  CharArrayType = ARRAY[0..15] OF Char;
CONST
   HexStr : CharArrayType = '0123456789ABCDEF';
BEGIN
  Assert(d < $78, 'error proc DecToHex');

  DecToHex := '$' + HexStr[d DIV 16] + HexStr[d MOD 16];
END;


FUNCTION OnBoard(Sq : Int16) : Boolean; InLine;
BEGIN
  OnBoard := (Sq AND $88) = 0;
END;


FUNCTION CoOrdOfFromSq(TheMove : Str4) : Int8; InLine;
BEGIN
  Assert( (TheMove[1] IN ['A'..'H']) AND (TheMove[2] IN ['1'..'8']), 'error proc CoOrdOfFromSq');

  CoOrdOfFromSq := 16*(Ord(TheMove[2]) - 49) + (Ord(TheMove[1]) - 65);
END;


FUNCTION CoOrdOfToSq(TheMove : Str4) : Int8; InLine;
BEGIN
  Assert( (TheMove[3] IN ['A'..'H']) AND (TheMove[4] IN ['1'..'8']), 'error proc CoOrdToOfToSq');

  CoOrdOfToSq := 16*(Ord(TheMove[4]) - 49) + (Ord(TheMove[3]) - 65);
END;


FUNCTION BoardSquareToString(Sq : Int8) : Str2; InLine;
{-- input is a board square 0..$77, output is coordinance string }
BEGIN
  Assert(Sq < $78, 'error proc BoardSquareToString');

  BoardSquareToString := Chr((Sq AND 7) + 65) + Chr((Sq SHR 4) + 49);
END;


FUNCTION Distance(Sq1, Sq2 : Int8) : Byte;
{-- manhattan distance between 2 squares }
VAR
  Vdist,
  Hdist : Byte;
BEGIN
  Assert((Sq1 < $78) AND (Sq2 <$78), 'Distance error');
  Assert(Sq1 <> Sq2, 'error Distance');

  IF (Sq1 = OffBoard) OR (Sq2 = OffBoard) THEN {-- fudge to make sure there is no bonus for king proximetry}
    Distance := 7
  ELSE
    BEGIN
      Vdist := Abs((Sq1 SHR 4) - (Sq2 SHR 4));
      Hdist := Abs((Sq1 AND 7) - (Sq2 AND 7));
      IF Vdist > Hdist THEN
        Distance := Vdist
      ELSE
        Distance := Hdist;
    END;
END;


FUNCTION Rank(Sq : SquareType) : Byte; InLine;
{-- returns from 0 to 7 }
BEGIN
  Assert(Sq < $78, 'error proc Rank ');

  IF (Sq <> OffBoard) THEN
    Rank := Sq SHR 4;
END;


FUNCTION Fyle(Sq : SquareType) : Byte; Inline;
{-- returns from 0 to 7 }
BEGIN
  Assert(Sq < $78, 'error proc Fyle');

  IF (Sq <> OffBoard) THEN
    Fyle := (Sq AND 7);
END;


FUNCTION DistanceToQueening(Sq : SquareType; Col : ColourType) : Byte;
BEGIN
  Assert(Sq < $78, 'error in Distance to queening');

  IF Col = WhitePieces THEN
    DistanceToQueening := 7 - Rank(Sq)
  ELSE
    DistanceToQueening := Rank(Sq);
END;


FUNCTION IsDiagonal(FromSq, ToSq : SquareType) : Boolean; InLine;
BEGIN
  Assert((FromSq < $78) AND (ToSq < $78), 'error in IsDiagonal');

  IsDiagonal := NOT (FromSq = OffBoard) AND NOT (ToSq = OffBoard) AND
                ( (Abs(ToSq - FromSq) MOD $F = 0) OR (Abs(ToSq - FromSq) MOD $11 = 0) );
END;


FUNCTION ClearVector(FromSq, ToSq : SquareType) : Boolean;
{-- is there amy pieces between fromsq and tosq...used for queen/rook/bishop moves }
VAR
  i,
  Step,
  Delta : Int8;
BEGIN
  Assert((FromSq < $78) AND (ToSq < $78), 'error in ClearVector');

  Delta := Abs(ToSq - FromSq);

  IF Delta MOD $F = 0 THEN      {-- diagonal }
    Step := $F
  ELSE
    IF Delta MOD $11 = 0 THEN   {-- diagonal }
      Step := $11
    ELSE
      IF Delta MOD $10 = 0 THEN {-- files north/south }
        Step := $10
      ELSE
        Step := 1;              {-- rank east/west }

  IF ToSq < FromSq THEN
    Step := -Step;

  i := FromSq;
  REPEAT
    Inc(i, Step);
  UNTIL (Data[Ply].Board[i] <> NoPiece) OR (i = ToSq);

  ClearVector := (i = ToSq);    {-- got there without bumping into a piece }
END;


FUNCTION ValidMove(M : Str4) : Boolean;
{-- used to validate transposition table move }
VAR
  FromSq,
  ToSq   : SquareType;
BEGIN
{-- poor mans move validator, since there are rarely hash errors and if the program doesnt crash with
    it, then it doesnt effect the performance }
  IF M = '' THEN
    ValidMove := False
  ELSE
    IF M = 'NULL' THEN
      ValidMove := True
    ELSE
      BEGIN
        FromSq := CoOrdOfFromSq(M);
        ToSq   := CoOrdOfToSq(M);

        WITH Data[Ply] DO
          IF Wtm THEN
            ValidMove := (Board[FromSq] IN WhitePieceSet) AND (Board[ToSq] IN GoodWhiteSquares)
          ELSE
            ValidMove := (Board[FromSq] IN BlackPieceSet) AND (Board[ToSq] IN GoodBlackSquares);
      END;
END;


FUNCTION Attacked(TheSq : SquareType; ByColor : ColourType) : Boolean;
{-- Is "TheSq" attacked by "ByColor" }
CONST
  PawnAtkDir   : SET OF SquareType = [$F, $11];
  KingAtkDir   : SET OF SquareType = [1, $10, $F, $11];
  KnightAtkDir : SET OF SquareType = [$E, $12, $1F, $21];
VAR
  i : 0..7;
BEGIN
  Attacked := False;              {-- unless proven otherwise }

  IF ByColor = BlackPieces THEN   {-- does black attack TheSq }
    BEGIN
      {-- queen attacks }
      IF Data[Ply].B_Queen[0] <> OffBoard THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].B_Queen[0])) AND ClearVector(TheSq, Data[Ply].B_Queen[0]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].B_Queen[0])) AND ClearVector(TheSq, Data[Ply].B_Queen[0]) THEN
            Exit(True);
          IF IsDiagonal(TheSq, Data[Ply].B_Queen[0]) AND ClearVector(TheSq, Data[Ply].B_Queen[0]) THEN
            Exit(True);
        END;
     IF Data[Ply].B_Queen[1] <> OffBoard THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].B_Queen[1])) AND ClearVector(TheSq, Data[Ply].B_Queen[1]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].B_Queen[1])) AND ClearVector(TheSq, Data[Ply].B_Queen[1]) THEN
            Exit(True);
          IF IsDiagonal(TheSq, Data[Ply].B_Queen[1]) AND ClearVector(TheSq, Data[Ply].B_Queen[1]) THEN
            Exit(True);
        END;

      {-- rook attack }
      IF Data[Ply].B_Rook[0] <> OffBoard THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].B_Rook[0])) AND ClearVector(TheSq, Data[Ply].B_Rook[0]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].B_Rook[0])) AND ClearVector(TheSq, Data[Ply].B_Rook[0]) THEN
            Exit(True);
        END;
      IF (Data[Ply].B_Rook[1] <> OffBoard) THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].B_Rook[1])) AND ClearVector(TheSq, Data[Ply].B_Rook[1]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].B_Rook[1])) AND ClearVector(TheSq, Data[Ply].B_Rook[1]) THEN
            Exit(True);
        END;

      {-- bishop attack }
      IF Data[Ply].B_Bishop[0] <> OffBoard THEN
        IF IsDiagonal(TheSq, Data[Ply].B_Bishop[0]) AND ClearVector(TheSq, Data[Ply].B_Bishop[0]) THEN
          Exit(True);
      IF (Data[Ply].B_Bishop[1] <> OffBoard) THEN
        IF IsDiagonal(TheSq, Data[Ply].B_Bishop[1]) AND ClearVector(TheSq, Data[Ply].B_Bishop[1]) THEN
          Exit(True);

       {-- knight attacks }
      IF Data[Ply].B_knight[0] <> OffBoard THEN
        IF Abs(TheSq - Data[Ply].B_Knight[0]) IN KnightAtkDir THEN
          Exit(True);
       IF Data[Ply].B_knight[1] <> OffBoard THEN
         IF Abs(TheSq - Data[Ply].B_Knight[1]) IN KnightAtkDir THEN
           Exit(True);

      {-- pawn attacks }
      FOR i := 0 TO 7 DO
        IF Data[Ply].B_Pawns[i] <> OffBoard THEN
          IF (Data[Ply].B_Pawns[i] - TheSq) IN PawnAtkDir THEN
            Exit(True);

      {-- king attack }
      IF Abs(TheSq - Data[Ply].B_King) IN KingAtkDir THEN
        Exit(True);
    END
  ELSE  {-- does white attack thesq }
    BEGIN
      {-- queen attacks }
      IF Data[Ply].W_Queen[0] <> OffBoard THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].W_Queen[0])) AND ClearVector(TheSq, Data[Ply].W_Queen[0]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].W_Queen[0])) AND ClearVector(TheSq, Data[Ply].W_Queen[0]) THEN
            Exit(True);
          IF IsDiagonal(TheSq, Data[Ply].W_Queen[0]) AND ClearVector(TheSq, Data[Ply].W_Queen[0]) THEN
            Exit(True);
        END;
     IF Data[Ply].W_Queen[1] <> OffBoard THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].W_Queen[1])) AND ClearVector(TheSq, Data[Ply].W_Queen[1]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].W_Queen[1])) AND ClearVector(TheSq, Data[Ply].W_Queen[1]) THEN
            Exit(True);
          IF IsDiagonal(TheSq, Data[Ply].W_Queen[1]) AND ClearVector(TheSq, Data[Ply].W_Queen[1]) THEN
            Exit(True);
        END;

      {-- rook attack }
      IF Data[Ply].W_Rook[0] <> OffBoard THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].W_Rook[0])) AND ClearVector(TheSq, Data[Ply].W_Rook[0]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].W_Rook[0])) AND ClearVector(TheSq, Data[Ply].W_Rook[0]) THEN
            Exit(True);
        END;
      IF (Data[Ply].W_Rook[1] <> OffBoard) THEN
        BEGIN
          IF (Rank(TheSq) = Rank(Data[Ply].W_Rook[1])) AND ClearVector(TheSq, Data[Ply].W_Rook[1]) THEN
            Exit(True);
          IF (Fyle(TheSq) = Fyle(Data[Ply].W_Rook[1])) AND ClearVector(TheSq, Data[Ply].W_Rook[1]) THEN
            Exit(True);
        END;

       {-- bishop attack }
      IF Data[Ply].W_Bishop[0] <> OffBoard THEN
        IF IsDiagonal(TheSq, Data[Ply].W_Bishop[0]) AND ClearVector(TheSq, Data[Ply].W_Bishop[0]) THEN
          Exit(True);
      IF (Data[Ply].W_Bishop[1] <> OffBoard) THEN
        IF IsDiagonal(TheSq, Data[Ply].W_Bishop[1]) AND ClearVector(TheSq, Data[Ply].W_Bishop[1]) THEN
          Exit(True);

       {-- knight attacks }
      IF Data[Ply].W_Knight[0] <> OffBoard THEN
        IF Abs(TheSq - Data[Ply].W_Knight[0]) IN KnightAtkDir THEN
          Exit(True);
       IF Data[Ply].W_Knight[1] <> OffBoard THEN
         IF Abs(TheSq - Data[Ply].W_Knight[1]) IN KnightAtkDir THEN
           Exit(True);

      {-- pawn attacks }
      FOR i := 0 TO 7 DO
        IF Data[Ply].W_Pawns[i] <> OffBoard THEN
          IF (TheSq - Data[Ply].W_Pawns[i]) IN PawnAtkDir THEN
            Exit(True);

      {-- king attack }
      IF Abs(TheSq - Data[Ply].W_King) IN KingAtkDir THEN
        Exit(True);
    END;
END;


FUNCTION InCheck(IsWhite : Boolean) : Boolean;
{-- is "color" in check }
BEGIN
  IF IsWhite THEN
    InCheck := Attacked(Data[Ply].W_King, BlackPieces)
  ELSE
    InCheck := Attacked(Data[Ply].B_King, WhitePieces);
END;


FUNCTION ThreeFoldDraw : Boolean;
{-- counts back the board positions (the TT key) to count how may positions repeat }
VAR
  i        : 0..MaxGameMoves;
  Total    : Byte;
  Position : Word64;
BEGIN
  Position := Data[Ply].ZorbistKey;           {-- the current position }

  Total := 0;
  FOR i := GameHistoryIndex DOWNTO 0 DO
    IF GameHistoryKeys[i] = Position THEN     {-- this position has occured before }
      Inc(Total);

  ThreeFoldDraw := Total = 3;
END;


PROCEDURE ErrorMsg(S : String);
BEGIN
  GoToXY(49, 14);
  ClrEol;
  Write(S);
END;


PROCEDURE IncMoveNmbr; InLine;
BEGIN
  IF MoveNmbr < MaxGameMoves THEN
    Inc(MoveNmbr)
  ELSE
    ErrorMsg('Exceeded Max Game Moves');
END;


PROCEDURE DecMoveNmbr; InLine;
BEGIN
  IF MoveNmbr > 1 THEN
    Dec(MoveNmbr);
END;


PROCEDURE SwapSides; InLine;
BEGIN
  Wtm        := NOT Wtm;
  HumansMove := NOT HumansMove;
END;


FUNCTION MovingPiece(Mstr : Str4) : PieceType; InLine;
BEGIN
  IF Mstr = '' THEN
    MovingPiece := NoPiece
  ELSE
    MovingPiece := Data[Ply].Board[CoOrdOfFromSq(Mstr)];
END;


FUNCTION ColourToMove(WhitesTurn : Boolean) : ColourType; InLine;
BEGIN
  ColourToMove := ColourType(Ord(WhitesTurn));
END;


PROCEDURE RemoveFromPieceList(Piece : PieceType; Sq : SquareType);
VAR
  i : 0..8;
BEGIN
  WITH Data[Ply] DO
    CASE Piece OF
      Wpawn   : BEGIN
                  i := 0;
                  WHILE (i < 8) and (W_Pawns[i] <> Sq) DO      {-- find the sq in the pawn list }
                    Inc(i);
                  IF (i < 8) then
                    W_Pawns[i] := OffBoard;
                END;

      Bpawn   : BEGIN
                  i := 0;
                  WHILE (i < 8) AND (B_Pawns[i] <> Sq) DO      {-- find the sq in the pawn list }
                    Inc(i);
                  IF (i < 8) then
                    B_Pawns[i] := OffBoard;
                END;

      Wrook   : IF W_Rook[0] = Sq THEN
                  W_Rook[0] := OffBoard
                ELSE
                  W_Rook[1] := OffBoard;

      Brook   : IF B_Rook[0] = Sq THEN
                  B_Rook[0] := OffBoard
                ELSE
                  B_Rook[1] := OffBoard;

      Wknight : IF W_Knight[0] = Sq THEN
                  W_Knight[0] := OffBoard
                ELSE
                   W_Knight[1] := OffBoard;

      Bknight : IF B_Knight[0] = Sq THEN
                  B_Knight[0] := OffBoard
                ELSE
                  B_Knight[1] := OffBoard;

      Wbishop : IF W_Bishop[0] = Sq THEN
                  W_Bishop[0] := OffBoard
                ELSE
                  W_Bishop[1] := OffBoard;

      Bbishop : IF B_Bishop[0] = Sq THEN
                  B_Bishop[0] := OffBoard
                ELSE
                   B_Bishop[1] := OffBoard;

      Wqueen  : IF W_Queen[0] = Sq THEN
                  W_Queen[0] := OffBoard
                ELSE
                  W_Queen[1] := OffBoard;

      Bqueen  : IF B_Queen[0] = Sq THEN
                  B_Queen[0] := OffBoard
                ELSE
                  B_Queen[1] := OffBoard;

      Wking   : W_King := OffBoard;

      Bking   : B_King := OffBoard;

    END;  {-- case }
END;


PROCEDURE AddToPieceList(Piece : PieceType; Sq : SquareType);
VAR
  i : 0..7;
BEGIN
  WITH Data[Ply] DO
    CASE Piece OF
      Wpawn   : BEGIN
                  i := 0;
                  WHILE W_Pawns[i] <> OffBoard DO  {-- find the first empty slot }
                    Inc(i);
                  W_Pawns[i] := Sq;
                END;

      Bpawn   : BEGIN
                  i := 0;
                  WHILE B_Pawns[i] <> OffBoard DO  {-- find the first empty slot }
                    Inc(i);
                  B_Pawns[i] := Sq;
                END;

      Wrook   : IF W_Rook[0] = OffBoard THEN
                  W_Rook[0] := Sq
                ELSE
                  W_Rook[1] := Sq;

      Brook   : IF B_Rook[0] = OffBoard THEN
                  B_Rook[0] := Sq
                ELSE
                  B_Rook[1] := Sq;

      Wknight : IF W_Knight[0] = OffBoard THEN
                  W_Knight[0] := Sq
                ELSE
                  W_Knight[1] := Sq;

      Bknight : IF B_Knight[0] = OffBoard THEN
                  B_Knight[0] := Sq
                ELSE
                  B_Knight[1] := Sq;

      Wbishop : IF W_Bishop[0] = OffBoard THEN
                  W_Bishop[0] := Sq
                ELSE
                  W_Bishop[1] := Sq;

      Bbishop : IF B_Bishop[0] = OffBoard THEN
                  B_Bishop[0] := Sq
                ELSE
                  B_Bishop[1] := Sq;

      Wqueen  : IF W_Queen[0] = OffBoard THEN
                  W_Queen[0] := Sq
                ELSE
                  W_Queen[1] := Sq;

      Bqueen  : IF B_Queen[0] = OffBoard THEN
                  B_Queen[0] := Sq
                ELSE
                  B_Queen[1] := Sq;

      Wking   : W_King := Sq;

      BKing   : B_King := Sq;

    END;  {-- case }
END;


PROCEDURE MoveStrToMoveRec(Mstr : Str4; VAR Mrec : MoveRecType);
{-- converts the 4 char move string to a full discription of the move }
BEGIN
  Assert(Mstr <> '', 'error Proc MoveStrToMoveRec');

  {-- decode the move string }
  WITH Mrec DO
    BEGIN
      FromSq        := CoOrdOfFromSq(MStr);
      ToSq          := CoOrdOfToSq(MStr);
      MovingPiece   := Data[Ply].Board[FromSq];
      CapturedPiece := Data[Ply].Board[ToSq];    {-- maybe no piece }

      Enpasaunt     := (Data[Ply].EnpasauntSq <> OffBoard) AND
                       ( (MovingPiece = Wpawn) AND (ToSq = Data[Ply].EnpasauntSq + $10) ) OR
                       ( (MovingPiece = Bpawn) AND (ToSq = Data[Ply].EnpasauntSq - $10) );

      Promotion     := ( (MovingPiece = WPawn) AND (ToSq > $69) ) OR ( (MovingPiece = BPawn) AND (ToSq < 8) );

      Castling      := ( (MovingPiece = Wking) AND ( (MStr = 'E1G1') OR (MStr = 'E1C1') ) ) OR
                       ( (MovingPiece = Bking) AND ( (MStr = 'E8G8') OR (MStr = 'E8C8') ) );
    END;
END;


PROCEDURE AddToKillers(Ply : Byte; TheMove : MoveType);
{-- killer moves caused beta cutoffs so we remember 2 per ply }
{-- Bump slot 1 down to slot 2, and update slot 1 }
VAR
  Mrec : MoveRecType;
BEGIN
  MoveStrToMoveRec(TheMove.MoveStr, Mrec);

  IF Mrec.CapturedPiece = NoPiece THEN  {-- captures come before killers so exclude these }
    IF TheMove.MoveValue > KillerMoves[0, Ply].MoveValue THEN
      BEGIN
        KillerMoves[1, Ply] := KillerMoves[0, Ply]; {-- put in second place }
        KillerMoves[0, Ply] := TheMove;             {-- new move in 1st place }
      END
    ELSE
      IF TheMove.MoveValue > KillerMoves[1, Ply].MoveValue THEN
        KillerMoves[1, Ply] := TheMove;             {-- put in second place }
END;


FUNCTION PVfromTT(D : Byte) : String;
{-- used for debugging }
VAR
  TTentry    : TTtype;
  TTstr      : String;
  SaveData   : DataType;
  SaveMovNbr : Byte;
  SaveHstIdx : Byte;
  Wmove,
  Hmove      : Boolean;
  i          : Byte;
BEGIN
  {-- save state of play }
  TTstr      := '';
  SaveData   := Data[Ply];
  SaveMovNbr := MoveNmbr;
  SaveHstIdx := GameHistoryIndex;
  Wmove      := Wtm;
  Hmove      := HumansMove;

  FOR i := 0 TO D DO
    BEGIN
      TTentry.Key := Data[Ply].ZorbistKey;
      GetFromTT(TTentry);
      IF TTentry.Node = Exact THEN
        BEGIN
          TTstr := TTstr + TTentry.Move + ' ';
          MakeMove(TTentry.Move, False);
        END;
    END;

  {-- restore state of play }
  Data[Ply]        := SaveData;
  MoveNmbr         := SaveMovNbr;
  GameHistoryIndex := SaveHstIdx;
  Wtm              := Wmove;
  HumansMove       := Hmove;

  PVfromTT := TTstr;
END;


FUNCTION WhiteMaterial : int16;
{-- but not pawns }
VAR
  i : 0..7;
  M : int16;
BEGIN
  M := 0;
  WITH Data[Ply] DO
    BEGIN
      FOR i := 0 TO 1 DO
        BEGIN
          IF W_Knight[i] <> OffBoard THEN
            Inc(M, PieceValue[WKnight]);
          IF W_Bishop[i] <> OffBoard THEN
            Inc(M, PieceValue[WBishop]);
          IF W_Rook[i] <> OffBoard THEN
            Inc(M, PieceValue[WRook]);
          IF W_Queen[i] <> OffBoard THEN
            Inc(M, PieceValue[WQueen]);
        END;
    END; {-- with }

  WhiteMaterial := M;
END;


FUNCTION BlackMaterial : int16;
{-- but not pawns }
{-- returns the absolute value }
VAR
  i : 0..7;
  M : int16;
BEGIN
  M := 0;
  WITH Data[Ply] DO
    BEGIN
      FOR i := 0 TO 1 DO
        BEGIN
          IF B_Knight[i] <> OffBoard THEN
            Inc(M, PieceValue[BKnight]);
          IF B_Bishop[i] <> OffBoard THEN
            Inc(M, PieceValue[BBishop]);
          IF B_Rook[i] <> OffBoard THEN
            Inc(M, PieceValue[BRook]);
          IF B_Queen[i] <> OffBoard THEN
            Inc(M, PieceValue[BQueen]);
        END;
    END; {-- with }

  BlackMaterial := M;
END;


FUNCTION NumOfPawns(Col : ColourType) : Byte;
VAR
  i   : 0..7;
  Num : 0..8;
BEGIN
  Num := 0;
  WITH Data[Ply] DO
  CASE Col OF
    WhitePieces :  FOR i := 0 TO 7 DO
                     IF W_Pawns[i] <> OffBoard THEN
                       Inc(Num);
    BlackPieces :  FOR i := 0 TO 7 DO
                     IF B_Pawns[i] <> OffBoard THEN
                       Inc(Num);
  END;

  NumOfPawns := Num;
END;


FUNCTION OnlyPawns(Wtm : Boolean) : Boolean;
BEGIN
  IF Wtm THEN
    OnlyPawns := WhiteMaterial = 0
  ELSE
    OnlyPawns := BlackMaterial = 0;
END;


FUNCTION CalcMaterial : int16;
{-- all pieces AND pawns, but not kings }
BEGIN
  CalcMaterial := WhiteMaterial - BlackMaterial +
                  PieceValue[WPawn]*NumOfPawns(WhitePieces) -
                  PieceValue[BPawn]*NumOfPawns(BlackPieces);
END;


FUNCTION OnlyKingsLeft : Boolean;
VAR
  i : Byte;
BEGIN
  OnlyKingsLeft := True;

  i := 0;
  WHILE (i < MaxSquares + 1) DO
    BEGIN
      IF NOT ( Data[Ply].Board[i] IN [NoPiece, Wking, Bking] )  THEN
        Exit(False);
      Inc(i);
    END;
END;


FUNCTION Castled(Col : ColourType) : Boolean; InLine;
BEGIN
  IF Col = WhitePieces THEN
    Castled := NOT(Data[Ply].WcanCastShort AND Data[Ply].WcanCastLong)
  ELSE
    Castled := NOT(Data[Ply].BcanCastShort AND Data[Ply].BcanCastLong);
END;


FUNCTION KnightOnTheRim(Sq : SquareType) : Boolean; InLine;
BEGIN
  KnightOnTheRim := Sq IN RimSqSet;
END;


PROCEDURE ClearBoard;
BEGIN
  FillChar(Data[Ply].Board, SizeOf(BoardType), 0);

  FillChar(Data[Ply].W_Pawns,  SizeOf(PawnListType),  OffBoard);
  FillChar(Data[Ply].B_Pawns,  SizeOf(PawnListType),  OffBoard);

  FillChar(Data[Ply].W_Rook,   SizeOf(PieceListType), OffBoard);
  FillChar(Data[Ply].B_Rook,   SizeOf(PieceListType), OffBoard);
  FillChar(Data[Ply].W_Knight, SizeOf(PieceListType), OffBoard);
  FillChar(Data[Ply].B_Knight, SizeOf(PieceListType), OffBoard);
  FillChar(Data[Ply].W_Bishop, SizeOf(PieceListType), OffBoard);
  FillChar(Data[Ply].B_Bishop, SizeOf(PieceListType), OffBoard);
  FillChar(Data[Ply].W_Queen,  SizeOf(PieceListType), OffBoard);
  FillChar(Data[Ply].B_Queen,  SizeOf(PieceListType), OffBoard);

  Data[Ply].W_King := OffBoard;
  Data[Ply].B_King := OffBoard;
END;


PROCEDURE WritePV(Len : Byte);
VAR
  i : Byte;
BEGIN
  Window(1, 1, 80, 25);

  GoToXY(2, 23);
  ClrEol;
  Write('PV:');

  FOR i := 0 TO 13 DO
    IF i <= Len THEN
      Write(PV[0, i] : 5);

  GoToXY(5, 24);
  ClrEol;
  FOR i := 14 TO Len DO
    Write(PV[0, i] : 5);
END;


PROCEDURE DebugPV;
VAR
  i, j : Byte;
BEGIN
  Window(1, 15, 80, 30);
  ClrScr;

  FOR i := 0 TO 9 DO
    BEGIN
      FOR j := 0 TO 9 DO
        Write(PV[i, j] : 5);
      Writeln;
    END;

  Readln;
  ClrScr;
END;


FUNCTION GetEndGame : Boolean;
BEGIN
  GetEndGame := ( (WhiteMaterial < PieceValue[WQueen]) AND (NumOfPawns(WhitePieces) < 5) ) OR
                ( (BlackMaterial < Abs(PieceValue[BQueen])) AND (NumOfPawns(BlackPieces) < 5) );
END;


PROCEDURE AlterPieceValues(C : Char);
BEGIN
  CASE C OF
   'W' : BEGIN
           PieceValue[WKnight] :=  350;
           PieceValue[WBishop] :=  375;
           PieceValue[WRook]   :=  575;
         END;

   'B' : BEGIN
           PieceValue[BKnight] := 350;
           PieceValue[BBishop] := 375;
           PieceValue[BRook]   := 575;
         END;

   'N' : BEGIN  {-- standard values }
           PieceValue[WPawn]   :=  100;
           PieceValue[WKnight] :=  300;
           PieceValue[WBishop] :=  325;
           PieceValue[WRook]   :=  525;
           PieceValue[WQueen]  :=  950;
           PieceValue[BPawn]   :=  100;
           PieceValue[BKnight] :=  300;
           PieceValue[BBishop] :=  325;
           PieceValue[BRook]   :=  525;
           PieceValue[BQueen]  :=  950;
         END;
  END;
END;



FUNCTION AverageEBF(VAR Nodes : SaveNodesType; Length : Byte) : Real;
{-- using the latest 2 ebf's  }
VAR
  Temp : ARRAY[0..1] OF Real;
BEGIN
  Assert(Length <= MaxPly, 'Error: searched too deep');

  IF Nodes[Length - 1] <> 0 THEN
    Temp[0] := (Nodes[Length] - Nodes[Length - 1]) / Nodes[Length - 1];  {-- now contains EBF per ply }
  IF Nodes[Length - 2] <> 0 THEN
    Temp[1] := (Nodes[Length - 1] - Nodes[Length - 2]) / Nodes[Length - 2];


  IF (Temp[0] + Temp[1] ) / 2  = 0 THEN
    AverageEBF := 2
  ELSE
    AverageEBF := (Temp[0] + Temp[1] ) / 2;
END;


FUNCTION InOpening : Boolean; Inline;
BEGIN
  InOpening := NOT InLibrary AND (MoveNmbr < 20);
END;


PROCEDURE AddToHistory(Move : Str4; Depth : int8);
VAR
  P : PieceType;
  S : SquareType;
BEGIN
  P := MovingPiece(Move);
  S := CoOrdOfToSq(Move);

  {-- add the move into the history table }
  Inc(HistoryTable[P, S], Depth*Depth);

  IF HistoryTable[P, S] > 190 THEN
    FOR P := WPawn TO BKing DO
      FOR S := 0 TO MaxSquares DO
        HistoryTable[P, S] := HistoryTable[P, S] DIV 2;
END;


PROCEDURE WriteHistoryTable;
{-- for debugging }
VAR
  f : Text;
  P : PieceType;
  S : SquareType;
BEGIN
  Assign(f, 'History.txt');
  ReWrite(f);

  FOR P := WPawn TO BKing DO
    FOR S := 0 TO MaxSquares DO
      IF HistoryTable[P, S] <> 0 THEN
        WriteLn(f, PieceTypeString[P], HistoryTable[P, S] : 4);

  Close(f);
END;



BEGIN
END.
