UNIT Zorbist;

INTERFACE
USES
  Globals;


TYPE   {-- a random number for every piece on every square }
  ZorbBrdType = ARRAY[PieceType, SquareType] OF Word64;
  PawnBrdType = ARRAY[SquareType] OF Word64;

VAR
  ZorbistBrd  : ZorbBrdType;   {-- the random numbers for each piece/colour/square }
  WhtPawnBrd,
  BlaPawnBrd  : PawnBrdType;


PROCEDURE InitZorbistBoard;    {-- values for each piece/square }

FUNCTION CalcBrdKey : Word64;  {-- used for init and error checking }

{-- for debugging }
FUNCTION CalcWhtPwnBrdKey : Word64;
FUNCTION CalcBlaPwnBrdKey : Word64;

{-----------------------------------------------------------------------------}
IMPLEMENTATION


PROCEDURE InitZorbistBoard;
{-- calculate a random 64 bit number for each piece on each square }
VAR
  Sq    : SquareType;
  Piece : PieceType;
BEGIN
  FillChar(ZorbistBrd, SizeOf(ZorbistBrd), 0);

  {-- calculate the entire table }
  {-- NoPiece has a zero value so will XOR to itself }
  Randomize;
  FOR Sq := 0 TO $77 DO
    FOR Piece := Wpawn TO Bking DO  {-- with each piece on it }
      BEGIN
        IF Piece = Wpawn THEN
          WhtPawnBrd[Sq] := (Word64(Random($FFFFFFFF)) SHL 32) OR
                             Word64(Random($FFFFFFFF));

        IF Piece = Bpawn THEN
          BlaPawnBrd[Sq] := (Word64(Random($FFFFFFFF)) SHL 32) OR
                             Word64(Random($FFFFFFFF));

        ZorbistBrd[Piece, Sq] := (Word64(Random($FFFFFFFF)) SHL 32) OR
                                  Word64(Random($FFFFFFFF));
      END;
END;


FUNCTION CalcBrdKey : Word64;
{-- calculate the zorist Key for the entire board }
VAR
  Rank, Fyle : 0..7;
  Value      : Word64;
  Sq         : Byte;
BEGIN
  Value := 0;
  FOR Rank := 0 TO 7 DO
    FOR Fyle := 0 TO 7 DO
      BEGIN
        Sq := $10*Rank + Fyle;
        Value := Value XOR ZorbistBrd[Data[Ply].Board[Sq], Sq];
      END;
  CalcBrdKey := Value;
END;


FUNCTION CalcWhtPwnBrdKey : Word64;
VAR
  Rank, Fyle : 0..7;
  Value      : Word64;
  Sq         : Byte;
BEGIN
  Value := 0;
  FOR Rank := 0 TO 7 DO
    FOR Fyle := 0 TO 7 DO
      BEGIN
        Sq := $10*Rank + Fyle;
        IF Data[Ply].Board[Sq] = Wpawn THEN
          Value := Value XOR WhtPawnBrd[Sq];
      END;
  CalcWhtPwnBrdKey := Value;
END;


FUNCTION CalcBlaPwnBrdKey : Word64;
VAR
  Rank, Fyle : 0..7;
  Value      : Word64;
  Sq         : Byte;
BEGIN
  Value := 0;
  FOR Rank := 0 TO 7 DO
    FOR Fyle := 0 TO 7 DO
      BEGIN
        Sq := $10*Rank + Fyle;
        IF Data[Ply].Board[Sq] = Bpawn THEN
          Value := Value XOR BlaPawnBrd[Sq];
      END;
  CalcBlaPwnBrdKey := Value;
END;



{-----------------------------------------------------------------------------}
BEGIN
END.


